#
# Copyright (c) 2015-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS MSC application.include module

This module is used for Pyramid integration
"""

import re

from pyams_app_msc.feature.tmdb.interfaces import TMDB_SEARCH_PATH, TMDB_SEARCH_ROUTE
from pyams_app_msc.interfaces import CANCEL_BOOKING_PERMISSION, CREATE_BOOKING_PERMISSION, MANAGE_BOOKING_PERMISSION, \
    MANAGE_CATALOG_PERMISSION, MANAGE_PLANNING_PERMISSION, MANAGE_THEATER_PERMISSION, MSC_CLIENT_ROLE, \
    MSC_CONTRIBUTOR_ROLE, MSC_MANAGER_ROLE, MSC_OPERATOR_ROLE, MSC_OWNER_ROLE, MSC_READER_ROLE, MSC_SITES_MANAGER_ROLE, \
    VIEW_BOOKING_PERMISSION, VIEW_CATALOG_PERMISSION, VIEW_PLANNING_PERMISSION, VIEW_THEATER_PERMISSION
from pyams_app_msc.shared.theater.api.interfaces import MSC_PRICE_API_PATH, MSC_PRICE_API_ROUTE
from pyams_content.interfaces import COMMENT_CONTENT_PERMISSION, CREATE_CONTENT_PERMISSION, CREATE_VERSION_PERMISSION, \
    MANAGE_CONTENT_PERMISSION, MANAGE_SITE_PERMISSION, MANAGE_SITE_TREE_PERMISSION, MANAGE_TOOL_PERMISSION, \
    PUBLISH_CONTENT_PERMISSION, WEBMASTER_ROLE
from pyams_layer.interfaces import MANAGE_SKIN_PERMISSION
from pyams_portal.interfaces import DESIGNER_ROLE
from pyams_security.interfaces.base import MANAGE_ROLES_PERMISSION, ROLE_ID, VIEW_SYSTEM_PERMISSION
from pyams_security.interfaces.names import ADMIN_USER_ID, SYSTEM_ADMIN_ROLE

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


def include_package(config):
    """Pyramid package include"""

    # add translations
    config.add_translation_dirs('pyams_app_msc:locales')

    # override PyAMS_content packages
    config.include('pyams_content_es')
    config.include('pyams_content_themes')
    config.include('pyams_content_api')

    # register permissions
    config.register_permission({
        'id': MANAGE_THEATER_PERMISSION,
        'title': _("Manage movie theater properties")
    })
    config.register_permission({
        'id': VIEW_THEATER_PERMISSION,
        'title': _("View movie theater properties")
    })
    config.register_permission({
        'id': MANAGE_CATALOG_PERMISSION,
        'title': _("Manage movie theater activities catalog")
    })
    config.register_permission({
        'id': VIEW_CATALOG_PERMISSION,
        'title': _("Read movie theater activities catalog")
    })
    config.register_permission({
        'id': CREATE_BOOKING_PERMISSION,
        'title': _("Create booking request for movie theater activity")
    })
    config.register_permission({
        'id': VIEW_PLANNING_PERMISSION,
        'title': _("View sessions planning on movie theater activities")
    })
    config.register_permission({
        'id': MANAGE_PLANNING_PERMISSION,
        'title': _("Manage sessions planning on movie theater activities")
    })
    config.register_permission({
        'id': VIEW_BOOKING_PERMISSION,
        'title': _("View booking information for movie theater activity")
    })
    config.register_permission({
        'id': MANAGE_BOOKING_PERMISSION,
        'title': _("Manage bookings for movie theater activity")
    })
    config.register_permission({
        'id': CANCEL_BOOKING_PERMISSION,
        'title': _("Cancel own bookings on movie theater activity")
    })

    # upgrade system manager roles
    config.upgrade_role(SYSTEM_ADMIN_ROLE,
                        permissions={
                            MANAGE_THEATER_PERMISSION, VIEW_THEATER_PERMISSION,
                            MANAGE_CATALOG_PERMISSION, VIEW_CATALOG_PERMISSION,
                            VIEW_PLANNING_PERMISSION, MANAGE_PLANNING_PERMISSION,
                            CREATE_BOOKING_PERMISSION, MANAGE_BOOKING_PERMISSION,
                            CANCEL_BOOKING_PERMISSION
                        })
    config.upgrade_role(WEBMASTER_ROLE,
                        managers={
                            ROLE_ID.format(MSC_SITES_MANAGER_ROLE)
                        })
    config.upgrade_role(DESIGNER_ROLE,
                        managers={
                            ROLE_ID.format(MSC_MANAGER_ROLE)
                        })

    # register new roles
    config.register_role({
        'id': MSC_SITES_MANAGER_ROLE,
        'title': _("MSC: Sites manager"),
        'permissions': {
            MANAGE_SITE_TREE_PERMISSION, MANAGE_ROLES_PERMISSION,
            VIEW_THEATER_PERMISSION, VIEW_SYSTEM_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE)
        }
    })
    config.register_role({
        'id': MSC_MANAGER_ROLE,
        'title': _("MSC: Theater manager"),
        'permissions': {
            MANAGE_THEATER_PERMISSION, VIEW_THEATER_PERMISSION,
            MANAGE_SITE_PERMISSION, MANAGE_SKIN_PERMISSION,
            MANAGE_TOOL_PERMISSION, MANAGE_CATALOG_PERMISSION,
            VIEW_CATALOG_PERMISSION, MANAGE_PLANNING_PERMISSION,
            VIEW_PLANNING_PERMISSION, VIEW_BOOKING_PERMISSION,
            MANAGE_BOOKING_PERMISSION, CANCEL_BOOKING_PERMISSION,
            VIEW_SYSTEM_PERMISSION, MANAGE_ROLES_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE),
            ROLE_ID.format(MSC_SITES_MANAGER_ROLE),
            ROLE_ID.format(MSC_MANAGER_ROLE)
        },
        'set_as_operator': False
    })
    config.register_role({
        'id': MSC_OPERATOR_ROLE,
        'title': _("MSC: Theater operator"),
        'permissions': {
            CREATE_CONTENT_PERMISSION, CREATE_VERSION_PERMISSION,
            MANAGE_CONTENT_PERMISSION, PUBLISH_CONTENT_PERMISSION,
            VIEW_THEATER_PERMISSION, MANAGE_CATALOG_PERMISSION,
            VIEW_CATALOG_PERMISSION, MANAGE_PLANNING_PERMISSION,
            VIEW_PLANNING_PERMISSION, VIEW_BOOKING_PERMISSION,
            MANAGE_BOOKING_PERMISSION, CANCEL_BOOKING_PERMISSION,
            VIEW_SYSTEM_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE),
            ROLE_ID.format(MSC_MANAGER_ROLE)
        },
        'set_as_operator': False
    })
    config.register_role({
        'id': MSC_CONTRIBUTOR_ROLE,
        'title': _("MSC: Contributor"),
        'permissions': {
            CREATE_CONTENT_PERMISSION, CREATE_VERSION_PERMISSION,
            VIEW_CATALOG_PERMISSION, MANAGE_CATALOG_PERMISSION,
            VIEW_PLANNING_PERMISSION, VIEW_BOOKING_PERMISSION,
            VIEW_SYSTEM_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE),
            ROLE_ID.format(MSC_MANAGER_ROLE),
            ROLE_ID.format(MSC_OPERATOR_ROLE)
        },
        'set_as_operator': False
    })
    config.register_role({
        'id': MSC_READER_ROLE,
        'title': _("MSC: Theater consultant"),
        'permissions': {
            COMMENT_CONTENT_PERMISSION, VIEW_THEATER_PERMISSION,
            VIEW_CATALOG_PERMISSION, VIEW_PLANNING_PERMISSION,
            VIEW_BOOKING_PERMISSION, VIEW_SYSTEM_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE),
            ROLE_ID.format(MSC_MANAGER_ROLE)
        },
        'set_as_operator': False
    })
    config.register_role({
        'id': MSC_CLIENT_ROLE,
        'title': _("MSC: Theater client"),
        'permissions': {
            VIEW_CATALOG_PERMISSION, CREATE_BOOKING_PERMISSION, CANCEL_BOOKING_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE)
        },
        'set_as_operator': False
    })
    config.register_role({
        'id': MSC_OWNER_ROLE,
        'title': _("MSC: Reservation owner"),
        'permissions': {
            VIEW_CATALOG_PERMISSION, MANAGE_BOOKING_PERMISSION, CANCEL_BOOKING_PERMISSION
        },
        'managers': {
            ADMIN_USER_ID,
            ROLE_ID.format(SYSTEM_ADMIN_ROLE)
        },
        'set_as_operator': False
    })

    # register custom REST API routes
    config.add_route(TMDB_SEARCH_ROUTE,
                     config.registry.settings.get(f'{TMDB_SEARCH_ROUTE}_route.path',
                                                  TMDB_SEARCH_PATH))

    config.add_route(MSC_PRICE_API_ROUTE,
                     config.registry.settings.get(f'{MSC_PRICE_API_ROUTE}_route.path',
                                                  MSC_PRICE_API_PATH))

    try:
        import pyams_zmi  # pylint: disable=import-outside-toplevel,unused-import
    except ImportError:
        config.scan(ignore=[re.compile(r'pyams_app_msc\..*\.zmi\.?.*').search])
    else:
        config.scan()



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.events import subscriber
from zope.interface import Interface, Invalid

from pyams_app_msc.shared.catalog import IWfCatalogEntry
from pyams_app_msc.shared.theater import IMovieTheater, IMovieTheaterSettings
from pyams_app_msc.shared.theater.interfaces import BOOKING_CANCEL_MODE
from pyams_content.interfaces import IBaseContent, MANAGE_SITE_TREE_PERMISSION
from pyams_content.root.zmi.sites import SiteRootSitesTable
from pyams_content.zmi.properties import PropertiesEditForm
from pyams_form.ajax import AJAXFormRenderer, ajax_form_config
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent, IFormContent, IGroup
from pyams_i18n.interfaces import II18n, INegotiator
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_skin.interfaces.viewlet import IBreadcrumbItem
from pyams_skin.viewlet.breadcrumb import BreadcrumbItem
from pyams_skin.viewlet.menu import MenuDivider, MenuItem
from pyams_table.interfaces import ITable
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.registry import get_utility
from pyams_utils.unicode import translate_string
from pyams_utils.url import absolute_url
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, AdminModalAddForm, FormGroupChecker
from pyams_zmi.helper.event import get_json_widget_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager, IMenuHeader, IPropertiesMenu, \
    ISiteManagementMenu
from pyams_zmi.table import TableElementEditor
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewlet_config(name='add-movie-theater.divider',
                context=ISiteRoot, layer=IAdminLayer, view=SiteRootSitesTable,
                manager=IContextAddingsViewletManager, weight=109,
                permission=MANAGE_SITE_TREE_PERMISSION)
class MovieTheaterAddMenuDivider(MenuDivider):
    """Movie theater add menu divider"""


@viewlet_config(name='add-movie-theater.menu',
                context=ISiteRoot, layer=IAdminLayer, view=SiteRootSitesTable,
                manager=IContextAddingsViewletManager, weight=110,
                permission=MANAGE_SITE_TREE_PERMISSION)
class MovieTheaterAddMenu(MenuItem):
    """Movie theater add menu"""

    label = _("Add movie theater")
    icon_class = 'fas fa-hotel'

    href = 'add-movie-theater.html'
    modal_target = True


@ajax_form_config(name='add-movie-theater.html',
                  context=ISiteRoot, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_TREE_PERMISSION)
class MovieTheaterAddForm(AdminModalAddForm):
    """Movie theater add form"""

    title = _("Add theater")
    legend = _("New theater properties")

    fields = Fields(IMovieTheater).select('title', 'short_name')
    content_factory = IMovieTheater

    _edit_permission = MANAGE_SITE_TREE_PERMISSION

    def add(self, obj):
        short_name = II18n(obj).query_attribute('short_name', request=self.request)
        name = translate_string(short_name, force_lower=True, spaces='-')
        self.context[name] = obj


@subscriber(IDataExtractedEvent, form_selector=MovieTheaterAddForm)
def handle_new_movie_theater_data(event):
    """Handle new movie theater data"""
    data = event.data
    negotiator = get_utility(INegotiator)
    name = data.get('short_name', {}).get(negotiator.server_language)
    if not name:
        event.form.widgets.errors += (Invalid(_("Movie theater name is required!")),)
    else:
        name = translate_string(name, force_lower=True, spaces='-')
        if name in event.form.context:
            event.form.widgets.errors += (Invalid(_("A movie theater is already registered "
                                                    "with this name!")),)


@adapter_config(required=(ISiteRoot, IAdminLayer, MovieTheaterAddForm),
                provides=IAJAXFormRenderer)
class MovieTheaterAddFormRenderer(ContextRequestViewAdapter):
    """Movie theater add form renderer"""

    def render(self, changes):
        if changes is None:
            return None
        return {
            'status': 'redirect',
            'location': absolute_url(changes, self.request, 'admin')
        }


@adapter_config(required=(IMovieTheater, IPyAMSLayer),
                provides=IObjectLabel)
@adapter_config(required=(IMovieTheater, IPyAMSLayer, Interface),
                provides=IObjectLabel)
def movie_theater_label(context, request, view=None):
    """Movie theater table element name"""
    return II18n(context).query_attribute('title', request=request)


@adapter_config(required=(IMovieTheater, IAdminLayer, ITable),
                provides=ITableElementEditor)
class MovieTheaterTableElementEditor(TableElementEditor):
    """Movie theater table element editor"""

    view_name = 'admin'
    modal_target = False


@adapter_config(required=(IMovieTheater, IAdminLayer, Interface),
                provides=IBreadcrumbItem)
class MovieTheaterBreadcrumbs(BreadcrumbItem):
    """Movie theater breadcrumb item"""

    @property
    def label(self):
        return II18n(self.context).query_attribute('short_name', request=self.request)

    css_class = 'breadcrumb-item persistent strong'
    view_name = 'admin'


@adapter_config(required=(IMovieTheater, IAdminLayer, Interface, ISiteManagementMenu),
                provides=IMenuHeader)
def movie_theater_management_menu_header(context, request, view, manager):
    """Movie theater management menu header adapter"""
    return _("Theater management")


@viewletmanager_config(name='properties.menu',
                       context=IMovieTheater, layer=IAdminLayer,
                       manager=ISiteManagementMenu, weight=10,
                       provides=IPropertiesMenu,
                       permission=VIEW_SYSTEM_PERMISSION)
class MovieTheaterPropertiesMenu(NavigationMenuItem):
    """Movie theater properties menu"""

    label = _("Properties")
    icon_class = 'fas fa-edit'
    href = '#properties.html'


@ajax_form_config(name='properties.html',
                  context=IMovieTheater, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class MovieTheaterPropertiesEditForm(PropertiesEditForm):
    """Movie theater properties edit form"""

    title = _("Movie theater properties")
    legend = _("Main theater properties")

    fields = Fields(IMovieTheater).select('title', 'short_name', 'header', 'description',
                                          'shared_content_workflow', 'logo', 'notepad',
                                          'address', 'banking_account', 'admin_info')


@adapter_config(name='theater-contacts',
                required=(IMovieTheater, IAdminLayer, MovieTheaterPropertiesEditForm),
                provides=IGroup)
class MovieTheaterPropertiesEditFormContactGroup(Group):
    """Movie theater contact group"""

    legend = _("Contacts")

    fields = Fields(IMovieTheater).select('web_address', 'contact_email', 'phone_number')
    weight = 1


@adapter_config(required=(IMovieTheater, IAdminLayer, MovieTheaterPropertiesEditForm),
                provides=IAJAXFormRenderer)
class MovieTheaterPropertiesEditFormRenderer(AJAXFormRenderer):
    """Movie theater properties edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        if 'title' in changes.get(IBaseContent, ()):
            return {
                'status': 'reload',
                'message': self.request.localizer.translate(self.form.success_message)
            }
        return super().render(changes)


#
# Movie theater settings edit form
#

@viewletmanager_config(name='settings.menu',
                       context=IMovieTheater, layer=IAdminLayer,
                       manager=IPropertiesMenu, weight=510,
                       permission=VIEW_SYSTEM_PERMISSION)
class MovieTheaterSettingsMenu(NavigationMenuItem):
    """Movie theater settings menu"""

    label = _("Settings")
    href = '#settings.html'


@ajax_form_config(name='settings.html',
                  context=IMovieTheater, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class MovieTheaterSettingsEditForm(AdminEditForm):
    """Movie theater settings edit form"""

    title = _("Movie theater settings")
    legend = _("Main theater settings")

    fields = Fields(IMovieTheaterSettings).select('calendar_first_day',
                                                  'calendar_slot_duration',
                                                  'default_session_duration',
                                                  'session_duration_delta')


@adapter_config(required=(IMovieTheater, IPyAMSLayer, MovieTheaterSettingsEditForm),
                provides=IFormContent)
def movie_theater_settings_edit_form_content(context, request, form):
    """Movie theater settings edit form content getter"""
    return IMovieTheaterSettings(context)


@adapter_config(name='',
                required=(IMovieTheater, IAdminLayer, MovieTheaterSettingsEditForm),
                provides=IGroup)
class MovieTheaterSettingsBookingGroup(Group):
    """Movie theater settings booking group"""

    legend = _("Booking settings")

    fields = Fields(IMovieTheaterSettings).select('reminder_delay',
                                                  'booking_retention_duration')
    weight = 10


@adapter_config(name='session-request-mode.group',
                required=(IMovieTheater, IAdminLayer, MovieTheaterSettingsEditForm),
                provides=IGroup)
class MovieTheaterSettingsSessionRequestModeGroup(FormGroupChecker):
    """Movie theater settings session request mode group"""

    fields = Fields(IMovieTheaterSettings).select('allow_session_request',
                                                  'session_request_mode')
    weight = 15


@adapter_config(name='booking-cancel-mode.group',
                required=(IMovieTheater, IAdminLayer, MovieTheaterSettingsEditForm),
                provides=IGroup)
class MovieTheaterSettingsBookingCancelModeGroup(Group):
    """Movie theater settings booking cancel mode"""

    legend = _("Booking cancel mode")

    fields = Fields(IMovieTheaterSettings).select('booking_cancel_mode',
                                                  'booking_cancel_max_delay',
                                                  'booking_cancel_notice_period')
    weight = 20


@subscriber(IDataExtractedEvent, form_selector=MovieTheaterSettingsBookingCancelModeGroup)
def handle_booking_cancel_mode(event):
    """Handle booking cancel mode"""
    data = event.data
    if (data.get('booking_cancel_mode') == BOOKING_CANCEL_MODE.MAX_DELAY.value) and \
            (not data.get('booking_cancel_max_delay')):
        event.form.widgets.errors += (Invalid(_("Cancel max delay must be set in \"max delay\" mode!")),)
    if (data.get('booking_cancel_mode') == BOOKING_CANCEL_MODE.NOTICE_PERIOD.value) and \
            (not data.get('booking_cancel_notice_period')):
        event.form.widgets.errors += (Invalid(_("Cancel notice period must be set in \"notice period\" mode!")),)


@adapter_config(name='quotations.group',
                required=(IMovieTheater, IAdminLayer, MovieTheaterSettingsEditForm),
                provides=IGroup)
class MovieTheaterSettingsQuotationsGroup(Group):
    """Movie theater quotations settings group"""

    legend = _("Quotations settings")

    fields = Fields(IMovieTheaterSettings).select('quotation_number_format', 'quotation_logo',
                                                  'quotation_color', 'currency', 'vat_rate')
    weight = 30


@adapter_config(required=(IMovieTheater, IAdminLayer, MovieTheaterSettingsQuotationsGroup),
                provides=IAJAXFormRenderer)
class MovieTheaterSettingsQuotationsGroupRenderer(AJAXFormRenderer):
    """Movie theater quotations settings group renderer"""

    def render(self, changes):
        result = super().render(changes)
        if 'quotation_logo' in changes.get(IMovieTheaterSettings, ()):
            result.setdefault('callbacks', []).append(
                get_json_widget_refresh_callback(self.form, 'quotation_logo', self.request))
        return result



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import implementer

__docformat__ = 'restructuredtext'

from pyams_app_msc.root.interfaces import IMSCSiteRootRoles, MSC_SITEROOT_ROLES
from pyams_security.interfaces import IRolesPolicy
from pyams_security.property import RolePrincipalsFieldProperty
from pyams_security.security import ProtectedObjectRoles
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import ContextAdapter, adapter_config


@implementer(IMSCSiteRootRoles)
class SiteRootRoles(ProtectedObjectRoles):
    """Site root roles"""

    msc_site_managers = RolePrincipalsFieldProperty(IMSCSiteRootRoles['msc_site_managers'])


@adapter_config(required=ISiteRoot,
                provides=IMSCSiteRootRoles)
def site_root_roles_adapter(context):
    """Site root roles adapter"""
    return SiteRootRoles(context)


@adapter_config(name=MSC_SITEROOT_ROLES,
                required=ISiteRoot,
                provides=IRolesPolicy)
class SiteRootRolesPolicy(ContextAdapter):
    """Site root roles policy"""

    roles_interface = IMSCSiteRootRoles
    weight = 50



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_app_msc.feature.tmdb.api module

This module provides an endpoint for querying the TMDB service.
"""

from cornice import Service
from cornice.validators import colander_validator
from pyramid.httpexceptions import HTTPOk

from pyams_app_msc.feature.tmdb import ITMDBService
from pyams_app_msc.feature.tmdb.api.schema import MovieSearchRequest, MovieSearchResponse
from pyams_app_msc.feature.tmdb.interfaces import TMDB_SEARCH_ROUTE
from pyams_security.interfaces.base import USE_INTERNAL_API_PERMISSION
from pyams_security.rest import check_cors_origin, set_cors_headers
from pyams_utils.factory import create_object
from pyams_utils.rest import STATUS, rest_responses

__docformat__ = 'restructuredtext'


tmdb_service = Service(name=TMDB_SEARCH_ROUTE,
                       pyramid_route=TMDB_SEARCH_ROUTE,
                       description="TMDB search service")


@tmdb_service.options(validators=(check_cors_origin, set_cors_headers))
def tmdb_service_options(request):
    """TMDB search service options handler"""


tmdb_service_get_responses = rest_responses.copy()
tmdb_service_get_responses[HTTPOk.code] = MovieSearchResponse(description="TMDB search response")


@tmdb_service.get(permission=USE_INTERNAL_API_PERMISSION,
                  schema=MovieSearchRequest(),
                  validators=(check_cors_origin, colander_validator, set_cors_headers),
                  response_schemas=tmdb_service_get_responses)
def find_movies(request):
    """TMDB movies finder"""
    params = request.validated.get('querystring', {})
    query = params.get('term')
    if not query:
        return {
            'status': STATUS.ERROR.value,
            'message': "Missing arguments",
            'results': []
        }
    service = create_object(ITMDBService)
    if (service is None) or (service.configuration is None):
        return {
            'status': STATUS.ERROR.value,
            'message': "Service unavailable",
            'results': []
        }
    return {
        'status': STATUS.SUCCESS.value,
        'results': sorted([
            {
                'id': f"movie_id::{movie['movie_id']}",
                'text': f"{movie['title']} ({movie['release_year']})" if ('release_year' in movie) else movie['title']
            }
            for movie in service.find_movies(query)
        ], key=lambda x: x['text'])
    }



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import implementer_only

from pyams_app_msc.component.duration.interfaces import IDurationField
from pyams_app_msc.component.duration.zmi.interfaces import IDurationWidget
from pyams_form.browser.text import TextWidget
from pyams_form.interfaces.widget import IFieldWidget
from pyams_form.widget import FieldWidget
from pyams_layer.interfaces import IFormLayer
from pyams_utils.adapter import adapter_config
from pyams_utils.interfaces.form import NO_VALUE

__docformat__ = 'restructuredtext'


@implementer_only(IDurationWidget)
class DurationWidget(TextWidget):
    """Duration widget"""

    def extract(self, default=NO_VALUE):
        value = super().extract(default)
        if isinstance(value, str):
            if ':' in value:
                hours, minutes = map(int, value.split(':'))
                value = str(hours * 60 + minutes)
            elif 'h' in value:
                hours, minutes = map(int, value.split('h'))
                value = str(hours * 60 + minutes)
            elif value:
                _value = int(value)
        return value


@adapter_config(required=(IDurationField, IFormLayer),
                provides=IFieldWidget)
def DurationFieldWidget(field, request):
    """Duration field widget"""
    return FieldWidget(field, DurationWidget(request))



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

import datetime
import json
from datetime import timezone

from pyramid.decorator import reify
from pyramid.events import subscriber
from pyramid.response import Response
from pyramid.view import view_config
from pyramid_mailer.message import Attachment
from zope.interface import Interface, Invalid, alsoProvides, implementer
from zope.lifecycleevent import ObjectModifiedEvent
from zope.principalannotation.interfaces import IPrincipalAnnotationUtility

from pyams_app_msc.feature.booking.interfaces import BOOKING_STATUS, BOOKING_STATUS_VOCABULARY, \
    IAcceptedBookingWorkflowInfo, IBookingAcceptInfo, IBookingContainer, IBookingInfo, IBookingTarget
from pyams_app_msc.feature.booking.message import get_booking_message, get_booking_message_values
from pyams_app_msc.feature.booking.zmi.dashboard import get_booking_element
from pyams_app_msc.feature.booking.zmi.interfaces import IBookingAcceptedStatusTable, IBookingContainerTable, \
    IBookingContainerView, IBookingForm, IBookingWaitingStatusTable
from pyams_app_msc.feature.messaging import IMessagingSettings
from pyams_app_msc.feature.planning.interfaces import ISession
from pyams_app_msc.feature.profile.interfaces import IOperatorProfile, IUserProfile, SEATS_DISPLAY_MODE, \
    USER_PROFILE_KEY
from pyams_app_msc.feature.profile.zmi.widget import PrincipalSelectFieldWidget
from pyams_app_msc.interfaces import MANAGE_BOOKING_PERMISSION, VIEW_BOOKING_PERMISSION
from pyams_app_msc.shared.theater.api.interfaces import MSC_PRICE_API_PATH, MSC_PRICE_API_ROUTE
from pyams_app_msc.shared.theater.interfaces import IMovieTheater, IMovieTheaterSettings
from pyams_app_msc.shared.theater.interfaces.mail import IMailTemplates
from pyams_app_msc.zmi import msc
from pyams_content.feature.history.interfaces import IHistoryContainer
from pyams_content.feature.history.zmi.viewlet import HistoryCommentsContentProvider
from pyams_content.zmi import content_js
from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.form import apply_changes
from pyams_form.group import Group
from pyams_form.interfaces import DISPLAY_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent, IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_mail.interfaces import IPrincipalMailInfo
from pyams_mail.message import HTMLMessage
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces import ISecurityManager
from pyams_security.utility import get_principal
from pyams_skin.interfaces.viewlet import IContentPrefixViewletManager, IFormHeaderViewletManager, IHelpViewletManager
from pyams_skin.schema.button import ActionButton, CloseButton, SubmitButton
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_skin.viewlet.help import AlertMessage
from pyams_table.column import GetAttrColumn
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.fanstatic import get_resource_path
from pyams_utils.interfaces import MISSING_INFO
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.interfaces.form import NO_VALUE_STRING
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalDisplayForm, AdminModalEditForm, \
    FormGroupChecker, SimpleAddFormRenderer, SimpleEditFormRenderer
from pyams_zmi.helper.container import delete_container_element
from pyams_zmi.helper.event import get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle, IModalDisplayFormButtons, check_submit_button
from pyams_zmi.interfaces.table import ITableElementEditor, ITableWithActions
from pyams_zmi.interfaces.viewlet import IToolbarViewletManager
from pyams_zmi.table import I18nColumnMixin, IconColumn, InnerTableAdminView, NameColumn, Table, TableElementEditor, \
    TrashColumn
from pyams_zmi.utils import get_object_hint, get_object_label

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@factory_config(IBookingContainerTable)
@implementer(ITableWithActions)
class BookingContainerTable(Table):
    """Booking target container table"""

    @reify
    def data_attributes(self):
        """Attributes getter"""
        attributes = super().data_attributes
        attributes.setdefault('table', {}).update({
            'data-ams-modules': json.dumps({
                'content': {
                    'src': get_resource_path(content_js)
                }
            })
        })
        return attributes

    display_if_empty = True


@adapter_config(required=(IBookingTarget, IAdminLayer, IBookingContainerTable),
                provides=IValues)
class BookingContainerTableValues(ContextRequestViewAdapter):
    """Booking container table values adapter"""

    @property
    def values(self):
        """Booking target container table values getter"""
        yield from IBookingContainer(self.context).values()


@adapter_config(name='creator',
                required=(IBookingContainer, IAdminLayer, IBookingContainerTable),
                provides=IColumn)
class BookingContainerCreatorColumn(NameColumn):
    """Booking container creator column"""

    i18n_header = _("Creator")
    weight = 10

    def get_value(self, obj):
        """Creator column value getter"""
        return get_principal(self.request, obj.creator).title


@adapter_config(name='recipient',
                required=(IBookingContainer, IAdminLayer, IBookingContainerTable),
                provides=IColumn)
class BookingContainerRecipientColumn(NameColumn):
    """Booking container recipient column"""

    i18n_header = _("Recipient")
    weight = 15

    def get_value(self, obj):
        """Recipient column value getter"""
        return get_principal(self.request, obj.recipient).title


@adapter_config(name='establishment',
                required=(IBookingContainer, IAdminLayer, IBookingContainerTable),
                provides=IColumn)
class BookingContainerEstablishmentColumn(NameColumn):
    """Booking container establishment column"""

    i18n_header = _("Establishment")
    weight = 20

    def get_value(self, obj):
        """Establishment column value getter"""
        principals = get_utility(IPrincipalAnnotationUtility)
        recipient = get_principal(self.request, obj.recipient)
        annotations = principals.getAnnotations(recipient)
        if annotations is None:
            return MISSING_INFO
        profile_info = IUserProfile(annotations.get(USER_PROFILE_KEY), None)
        if profile_info is None:
            return MISSING_INFO
        result = profile_info.establishment or MISSING_INFO
        if obj.cultural_pass:
            label = self.request.localizer.translate(_("Cultural pass"))
            result += (f' <img title="{label}" alt="" '
                       f'      class="mx-1 mt-n1 hint" '
                       f'      src="/--static--/msc/img/pass-culture.webp" '
                       f'      width="24" height="24" />')
        return result


@adapter_config(name='status',
                required=(IBookingContainer, IAdminLayer, IBookingContainerTable),
                provides=IColumn)
class BookingContainerStatusColumn(I18nColumnMixin, GetAttrColumn):
    """Booking container status column"""

    i18n_header = _("Status")
    weight = 30

    def get_value(self, obj):
        """Status column value getter"""
        translate = self.request.localizer.translate
        status = BOOKING_STATUS_VOCABULARY.by_value.get(obj.status)
        return translate(status.title) if status is not None else MISSING_INFO


@adapter_config(name='seats',
                required=(IBookingContainer, IAdminLayer, IBookingContainerTable),
                provides=IColumn)
class BookingContainerSeatsColumn(I18nColumnMixin, GetAttrColumn):
    """Booking container seats column"""

    i18n_header = _("Seats")
    weight = 40

    def get_value(self, obj):
        """Seats column value getter"""
        return f'{obj.nb_participants} + {obj.nb_accompanists}'


@adapter_config(name='quotation',
                required=(IBookingContainer, IAdminLayer, IBookingContainerTable),
                provides=IColumn)
class BookingContainerQuotationColumn(IconColumn):
    """Booking container quotation column"""

    icon_class = 'fas fa-file-pdf'
    hint = _("Quotation")
    weight = 45

    @staticmethod
    def checker(obj):
        return (obj.status == BOOKING_STATUS.ACCEPTED.value) and obj.quotation

    def render_cell(self, item):
        result = super().render_cell(item)
        if result:
            quotation = item.quotation
            return (f'<a href="{absolute_url(quotation, self.request)}"'
                    f'   target="_blank">{result}</a>')
        return result


@adapter_config(name='archive',
                required=(IBookingContainer, IAdminLayer, IBookingContainerTable),
                provides=IColumn)
class BookingContainerArchiveColumn(IconColumn):
    """Booking container archive column"""

    css_classes = {
        'th': 'action',
        'td': 'text-danger'
    }
    icon_class = 'fas fa-archive'
    hint = _("Archived booking")
    weight = 50

    def checker(self, item):
        return IBookingInfo(item).archived


@adapter_config(name='trash',
                required=(IBookingContainer, IAdminLayer, IBookingContainerTable),
                provides=IColumn)
class BookingContainerTrashColumn(TrashColumn):
    """Booking container trash column"""

    object_data = {
        'ams-modules': 'container',
        'ams-delete-target': 'delete-booking.json'
    }


@view_config(name='delete-booking.json',
             context=IBookingContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=MANAGE_BOOKING_PERMISSION)
def delete_booking(request):
    """Delete booking"""
    return delete_container_element(request, container_factory=IBookingContainer)


@viewlet_config(name='bookings-table',
                context=IBookingTarget, layer=IAdminLayer,
                view=IBookingContainerView,
                manager=IContentPrefixViewletManager, weight=10)
class BookingContainerTableView(InnerTableAdminView):
    """Booking target container table view"""

    table_class = IBookingContainerTable
    table_label = _("Current session bookings")

    container_intf = IBookingContainer


@pagelet_config(name='bookings.html',
                context=IBookingTarget, layer=IPyAMSLayer,
                permission=VIEW_BOOKING_PERMISSION)
@implementer(IBookingContainerView)
class BookingContainerModalView(AdminModalDisplayForm):
    """Booking target container modal view"""

    @property
    def subtitle(self):
        """Subtitle getter"""
        translate = self.request.localizer.translate
        subtitle = translate(_("Session bookings"))
        profile = IOperatorProfile(self.request)
        if profile.session_seats_display_mode != SEATS_DISPLAY_MODE.NONE.value:
            container = IBookingContainer(self.context)
            if container is not None:
                subtitle += translate(_(" (Seats: {})")).format(container.get_seats(profile.session_seats_display_mode))
        return subtitle

    modal_class = 'modal-xl'


@adapter_config(required=(IBookingTarget, IAdminLayer, IBookingContainerView),
                provides=IFormTitle)
def booking_target_edit_form_title(context, request, form):
    """Booking target edit form title"""
    translate = request.localizer.translate
    theater = get_parent(context, IMovieTheater)
    hint = get_object_hint(context, request, form)
    label = get_object_label(context, request, form)
    return TITLE_SPAN_BREAK.format(
        get_object_label(theater, request, form),
        translate(_("{}: {}")).format(hint, label) if hint else label)


#
# Booking add and edit forms
#

@adapter_config(required=(IBookingInfo, IAdminLayer, Interface),
                provides=IObjectLabel)
def get_booking_label(context, request, view):
    """Booking label getter"""
    recipient = get_principal(request, context.recipient)
    result = recipient.title
    profile_info = IUserProfile(recipient, None)
    if (profile_info is None) or not profile_info.establishment:
        return result
    return (f'{result} ({profile_info.get_structure_type()} - '
            f'{profile_info.establishment}, {profile_info.establishment_address.city} - '
            f'{profile_info.phone_number})')


@viewlet_config(name='add-booking.action',
                context=IBookingTarget, layer=IAdminLayer,
                view=IBookingContainerTable,
                manager=IToolbarViewletManager, weigth=20,
                permission=MANAGE_BOOKING_PERMISSION)
class BookingAddAction(ContextAddAction):
    """Booking add action"""

    label = _("Add booking")
    href = 'add-booking.html'

    def __new__(cls, context, request, view, manager):
        session = ISession(context, None)
        if (session is not None) and not session.bookable:
            return None
        return ContextAddAction.__new__(cls)

    def get_href(self):
        return absolute_url(self.context, self.request, f'++booking++/{self.href}')


# class IBookingAddFormButtons(IModalAddFormButtons):
#     """Booking add form buttons"""
#
#     validate = SubmitButton(name='validate',
#                             title=_("Add and accept"),
#                             condition=check_submit_button)


@ajax_form_config(name='add-booking.html',
                  context=IBookingContainer, layer=IPyAMSLayer,
                  permission=MANAGE_BOOKING_PERMISSION)
class BookingAddForm(AdminModalAddForm):
    """Booking add form"""

    modal_class = 'modal-xl'

    subtitle = _("New booking")
    legend = _("New booking properties")

    fields = Fields(IBookingInfo).select('recipient', 'status',
                                         'nb_participants', 'nb_accompanists', 'nb_seats',
                                         'nb_free_accompanists', 'accompanying_ratio',
                                         'nb_groups', 'price', 'cultural_pass',
                                         'comments', 'notepad')
    fields['recipient'].widget_factory = PrincipalSelectFieldWidget
    # buttons = Buttons(IBookingAddFormButtons).select('add', 'validate', 'close')

    content_factory = IBookingInfo

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        status = self.widgets.get('status')
        if status is not None:
            status.mode = DISPLAY_MODE
        nb_seats = self.widgets.get('nb_seats')
        if nb_seats is not None:
            nb_seats.mode = DISPLAY_MODE
            seats_data = {
                'ams-change-handler': 'MyAMS.msc.booking.seatsChanged',
                'ams-change-handler-options': {
                    'target': nb_seats.id
                }
            }
            nb_participants = self.widgets.get('nb_participants')
            if nb_participants is not None:
                nb_participants.object_data = seats_data
                alsoProvides(nb_participants, IObjectData)
            nb_accompanists = self.widgets.get('nb_accompanists')
            if nb_accompanists is not None:
                nb_accompanists.object_data = seats_data
                alsoProvides(nb_accompanists, IObjectData)
        price = self.widgets.get('price')
        if price is not None:
            theater = get_parent(self.context, IMovieTheater)
            price.object_data = {
                'ams-modules': {
                    'msc': {
                        'src': get_resource_path(msc)
                    }
                },
                'ams-change-handler': 'MyAMS.helpers.select2ChangeHelper',
                'ams-select2-helper-url':
                    absolute_url(self.request.root, self.request) +
                    self.request.registry.settings.get(f'{MSC_PRICE_API_ROUTE}_route.path',
                                                       MSC_PRICE_API_PATH).format(
                        theater_name=theater.__name__),
                'ams-select2-helper-argument': 'price_id',
                'ams-select2-helper-callback': 'MyAMS.msc.booking.priceChanged'
            }
            alsoProvides(price, IObjectData)

    def update_actions(self):
        super().update_actions()
        validate = self.actions.get('validate')
        if validate is not None:
            validate.add_class('btn-info')

    def update_content(self, obj, data):
        request = self.request
        form_data = data.get(self, data)
        notepad = form_data.get('notepad')
        message = form_data.get('notify_message') if form_data.get('notify_recipient') else None
        if notepad or message:
            history = IHistoryContainer(obj, None)
            if history is not None:
                history.add_history(obj,
                                    comment=notepad,
                                    message=message,
                                    request=request)
        changes = apply_changes(self, obj, form_data)
        obj.creator = request.principal.id
        accepted = form_data.get('accepted', False)
        if accepted:
            obj.status = BOOKING_STATUS.ACCEPTED.value
            if 'quotation_message' in form_data:
                obj.quotation_message = form_data['quotation_message']
            request.registry.notify(ObjectModifiedEvent(obj))
            if message:
                settings = IMessagingSettings(request.root, None)
                if settings is None:
                    return changes
                mailer = settings.get_mailer()
                if mailer is None:
                    return changes
                html_message = get_booking_message(_("Booking accepted"), form_data,
                                                   obj, request, settings)
                if html_message is not None:
                    mailer.send(html_message)
        return changes

    def add(self, obj):
        IBookingContainer(self.context).append(obj)


@adapter_config(required=(IBookingContainer, IAdminLayer, BookingAddForm),
                provides=IFormTitle)
def booking_add_form_title(context, request, form):
    """Booking add form title"""
    session = get_parent(context, ISession)
    return booking_target_edit_form_title(session, request, form)


@adapter_config(name='validate.group',
                required=(IBookingContainer, IAdminLayer, BookingAddForm),
                provides=IGroup)
class BookingAddFormValidateGroup(FormGroupChecker):
    """Booking add form validate group"""

    fields = Fields(IBookingAcceptInfo).select('accepted')
    weight = 20


@subscriber(IDataExtractedEvent, form_selector=BookingAddFormValidateGroup)
def handle_validate_group_data(event):
    """Handle validate group data"""
    data = event.data
    if data.get('accepted'):
        parent_form = event.form.parent_form
        if NO_VALUE_STRING in parent_form.widgets['price'].value:
            event.form.widgets.errors += (Invalid(_("You must set a price to accept a booking!")),)


@adapter_config(name='notify.group',
                required=(IBookingContainer, IAdminLayer, BookingAddFormValidateGroup),
                provides=IGroup)
class BookingAddFormNotifyGroup(FormGroupChecker):
    """Booking add form notify group"""

    fields = Fields(IAcceptedBookingWorkflowInfo).select('notify_recipient', 'notify_subject',
                                                         'notify_message')
    weight = 20

    def update_widgets(self, prefix=None, use_form_mode=True):
        super().update_widgets(prefix, use_form_mode)
        notify = self.widgets.get('notify_recipient')
        if notify is not None:
            notify.value = ()
        # generate formatted messages
        theater = get_parent(self.context, IMovieTheater)
        templates = IMailTemplates(theater)
        values = get_booking_message_values(self.context, self.request, self)
        subject = self.widgets.get('notify_subject')
        if subject is not None:
            template = getattr(templates, 'accept_subject')
            if template:
                subject.value = template.format(**values)
        message = self.widgets.get('notify_message')
        if message is not None:
            template = getattr(templates, f'accept_template')
            if template:
                message.value = template.format(**values)


@adapter_config(name='quotation.group',
                required=(IBookingContainer, IAdminLayer, BookingAddFormNotifyGroup),
                provides=IGroup)
class BookingAddFormQuotationGroup(FormGroupChecker):
    """Booking add form quotation group"""

    fields = Fields(IAcceptedBookingWorkflowInfo).select('include_quotation', 'quotation_message')
    weight = 10

    def update_widgets(self, prefix=None, use_form_mode=True):
        super().update_widgets(prefix, use_form_mode)
        include_quotation = self.widgets.get('include_quotation')
        if include_quotation is not None:
            include_quotation.value = ()


@adapter_config(required=(IBookingContainer, IAdminLayer, BookingAddForm),
                provides=IAJAXFormRenderer)
class BookingAddFormRenderer(SimpleAddFormRenderer):
    """Booking add form renderer"""

    table_factory = IBookingContainerTable

    def render(self, changes):
        result = super().render(changes)
        if changes:
            result.setdefault('callbacks', []).append({
                'callback': 'MyAMS.msc.calendar.refresh',
                'options': {
                    'room_id': ISession(self.context).room
                }
            })
        return result


@adapter_config(required=(IBookingInfo, IAdminLayer, IBookingContainerTable),
                provides=ITableElementEditor)
class BookingInfoEditor(TableElementEditor):
    """Booking info editor"""


@adapter_config(required=(IBookingInfo, IAdminLayer, IBookingForm),
                provides=IFormTitle)
def booking_edit_form_title(context, request, form):
    """Booking workflow management form title"""
    theater = get_parent(context, IMovieTheater)
    session = get_parent(context, ISession)
    return TITLE_SPAN_BREAK.format(
        get_object_label(theater, request, form),
        get_object_label(session, request, form))


class IBookingEditFormButtons(Interface):
    """Booking edit form buttons"""

    preview_quotation = ActionButton(name='preview_quotation',
                                     title=_("Quotation preview"))

    reset_quotation = ActionButton(name='reset_quotation',
                                   title=_("Reset quotation"),
                                   condition=lambda form:
                                       (form.context.status == BOOKING_STATUS.ACCEPTED.value) and
                                       check_submit_button(form))

    apply = SubmitButton(name='apply',
                         title=_("Apply"),
                         condition=check_submit_button)

    close = CloseButton(name='close',
                        title=_("Cancel"))


@ajax_form_config(name='properties.html',
                  context=IBookingInfo, layer=IPyAMSLayer,
                  permission=VIEW_BOOKING_PERMISSION)
@implementer(IBookingForm)
class BookingPropertiesEditForm(AdminModalEditForm):
    """Booking info properties edit form"""

    @property
    def subtitle(self):
        translate = self.request.localizer.translate
        return translate(_("Booking: {}")).format(
            get_object_label(self.context, self.request, self))

    modal_class = 'modal-xl'
    legend = _("Booking properties")

    fields = Fields(IBookingInfo).select('creator', 'recipient', 'status',
                                         'nb_participants', 'nb_accompanists', 'nb_seats',
                                         'nb_free_accompanists', 'accompanying_ratio',
                                         'nb_groups', 'price', 'cultural_pass',
                                         'comments', 'notepad')
    fields['recipient'].widget_factory = PrincipalSelectFieldWidget

    @reify
    def buttons(self):
        if self.mode == DISPLAY_MODE:
            return Buttons(IModalDisplayFormButtons)
        return Buttons(IBookingEditFormButtons)

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        creator = self.widgets.get('creator')
        if creator is not None:
            creator.mode = DISPLAY_MODE
        status = self.widgets.get('status')
        if status is not None:
            status.mode = DISPLAY_MODE
        nb_seats = self.widgets.get('nb_seats')
        if nb_seats is not None:
            nb_seats.mode = DISPLAY_MODE
            seats_data = {
                'ams-change-handler': 'MyAMS.msc.booking.seatsChanged',
                'ams-change-handler-options': {
                    'target': nb_seats.id
                }
            }
            nb_participants = self.widgets.get('nb_participants')
            if nb_participants is not None:
                nb_participants.object_data = seats_data
                alsoProvides(nb_participants, IObjectData)
            nb_accompanists = self.widgets.get('nb_accompanists')
            if nb_accompanists is not None:
                nb_accompanists.object_data = seats_data
                alsoProvides(nb_accompanists, IObjectData)
        price = self.widgets.get('price')
        if price is not None:
            theater = get_parent(self.context, IMovieTheater)
            price.object_data = {
                'ams-modules': {
                    'msc': {
                        'src': get_resource_path(msc)
                    }
                },
                'ams-change-handler': 'MyAMS.helpers.select2ChangeHelper',
                'ams-select2-helper-url':
                    absolute_url(self.request.root, self.request) +
                    self.request.registry.settings.get(f'{MSC_PRICE_API_ROUTE}_route.path',
                                                       MSC_PRICE_API_PATH).format(
                        theater_name=theater.__name__),
                'ams-select2-helper-argument': 'price_id',
                'ams-select2-helper-callback': 'MyAMS.msc.booking.priceChanged'
            }
            alsoProvides(price, IObjectData)
        notepad = self.widgets.get('notepad')
        if notepad is not None:
            history = HistoryCommentsContentProvider(self.context, self.request, self)
            history.update()
            notepad.prefix = history
            if f'{self.prefix}{self.widgets.prefix}notepad' not in self.request.params:
                notepad.value = ''

    def update_actions(self):
        super().update_actions()
        preview_quotation = self.actions.get('preview_quotation')
        reset_quotation = self.actions.get('reset_quotation')
        if preview_quotation is not None:
            preview_quotation.add_class('btn-info')
            if reset_quotation is None:
                preview_quotation.add_class('mr-auto')
            preview_quotation.object_data = {
                'ams-click-handler': 'MyAMS.msc.booking.previewQuotation',
                'ams-click-handler-options': {
                    'target': absolute_url(self.context, self.request, 'preview-quotation.pdf')
                }
            }
            alsoProvides(preview_quotation, IObjectData)
        if reset_quotation is not None:
            reset_quotation.add_class('btn-info mr-auto')
            reset_quotation.object_data = {
                'ams-click-handler': 'MyAMS.msc.booking.resetQuotation',
                'ams-click-handler-options': {
                    'target': absolute_url(self.context, self.request, 'reset-quotation.json')
                }
            }
            alsoProvides(reset_quotation, IObjectData)

    @handler(IBookingEditFormButtons['apply'])
    def handle_apply(self, action):
        return super().handle_apply(self, action)

    def apply_changes(self, data):
        form_data = data.get(self, data)
        update_message = form_data.get('update_message') if form_data.get('send_update') else None
        history = IHistoryContainer(self.context, None)
        if history is not None:
            history.add_history(self.context,
                                comment=form_data.get('notepad'),
                                request=self.request)
        result = super().apply_changes(data)
        if result and update_message:
            settings = IMessagingSettings(self.request.root, None)
            if settings is None:
                return
            mailer = settings.get_mailer()
            if mailer is None:
                return
            html_message = self.get_message(form_data, self.context, self.request, settings)
            if html_message is not None:
                mailer.send(html_message)
        return result

    def get_message(self, data, context, request, settings):
        sm = get_utility(ISecurityManager)
        principal = sm.get_raw_principal(context.recipient)
        mail_info = IPrincipalMailInfo(principal, None)
        if mail_info is None:
            return
        mail_addresses = [
            f'{name} <{address}>'
            for name, address in mail_info.get_addresses()
        ]
        subject = data.get('update_subject') or context.update_subject
        cc = None
        theater = get_parent(context, IMovieTheater)
        templates = IMailTemplates(theater)
        if templates.send_copy_to_sender:
            principal = request.principal
            if principal is not None:
                mail_info = IPrincipalMailInfo(principal, None)
                if mail_info is not None:
                    cc = [
                        f'{name} <{address}>'
                        for name, address in mail_info.get_addresses()
                    ]
        message = HTMLMessage(f'{settings.subject_prefix} {subject}',
                              from_addr=f'{settings.source_name} <{settings.source_address}>',
                              to_addr=mail_addresses,
                              cc=cc,
                              html=data.get('update_message'))
        quotation = context.get_quotation(force_refresh=True)
        if quotation is not None:
            message.attach(Attachment(content_type='application/pdf',
                                      data=str(quotation),
                                      filename=f'{context.quotation_number}.pdf'))
        return message


@viewlet_config(name='booking-archive.warning',
                context=IBookingInfo, layer=IAdminLayer, view=BookingPropertiesEditForm,
                manager=IFormHeaderViewletManager, weight=10)
class BookingInfoArchiveWarning(AlertMessage):
    """Booking archive warning"""

    def __new__(cls, context, request, view, manager):
        if not context.archived:
            return None
        return AlertMessage.__new__(cls)

    status = 'warning'
    _message = _("This booking is archived and can't be modified anymore!")


@adapter_config(required=(IBookingInfo, IAdminLayer, BookingPropertiesEditForm),
                provides=IAJAXFormRenderer)
class BookingPropertiesEditFormRenderer(SimpleEditFormRenderer):
    """Booking properties edit form renderer"""

    parent_interface = IBookingContainer
    table_factory = IBookingContainerTable

    def render(self, changes):
        result = super().render(changes)
        if changes:
            theater = get_parent(self.context, IMovieTheater)
            result.setdefault('callbacks', []).extend([
                get_json_table_row_refresh_callback(theater, self.request,
                                                    IBookingWaitingStatusTable, get_booking_element(self.context)),
                get_json_table_row_refresh_callback(theater, self.request,
                                                    IBookingAcceptedStatusTable, get_booking_element(self.context)),
                {
                    'callback': 'MyAMS.msc.calendar.refresh',
                    'options': {
                        'room_id': ISession(self.context).room
                    }
                }
            ])
        return result


@subscriber(IDataExtractedEvent, form_selector=BookingAddForm)
@subscriber(IDataExtractedEvent, form_selector=BookingPropertiesEditForm)
def handle_accepted_booking_data(event):
    """Handle accepted booking form data"""
    data = event.data
    if (data.get('status') == BOOKING_STATUS.ACCEPTED.value) and not data.get('price'):
        event.form.widgets.errors += (Invalid(_("You must set a price to accept a booking!")),)


@adapter_config(name='session.group',
                required=(IBookingContainer, IAdminLayer, BookingAddForm),
                provides=IGroup)
@adapter_config(name='session.group',
                required=(IBookingInfo, IAdminLayer, BookingPropertiesEditForm),
                provides=IGroup)
class BookingFormSessionGroup(Group):
    """Session info display form"""

    legend = _("Session properties")
    fields = Fields(ISession).select('capacity') + Fields(IBookingContainer).select('free_seats')

    _mode = DISPLAY_MODE
    weight = 10

    def update_widgets(self, prefix=None, use_form_mode=True):
        super().update_widgets(prefix, use_form_mode)
        session = get_parent(self.context, ISession)
        capacity = self.widgets.get('capacity')
        if capacity is not None:
            capacity.value = ISession(session).capacity
        free_seats = self.widgets.get('free_seats')
        if free_seats is not None:
            free_seats.value = IBookingContainer(session).free_seats


@adapter_config(name='quotation.group',
                required=(IBookingInfo, IAdminLayer, BookingPropertiesEditForm),
                provides=IGroup)
class BookingEditFormQuotationGroup(Group):
    """Quotation info display form"""

    def __new__(cls, context, request, form):
        if context.status != BOOKING_STATUS.ACCEPTED.value:
            return None
        return Group.__new__(cls)

    legend = _("Booking quotation")
    fields = Fields(IBookingInfo).select('quotation_number', 'quotation_message', 'quotation')

    _mode = DISPLAY_MODE
    weight = 20


@adapter_config(name='update-group',
                required=(IBookingInfo, IAdminLayer, BookingPropertiesEditForm),
                provides=IGroup)
class BookingEditFormUpdateGroup(FormGroupChecker):
    """Booking edit form update group"""

    def __new__(cls, context, request, form):
        if context.status != BOOKING_STATUS.ACCEPTED.value:
            return None
        return FormGroupChecker.__new__(cls)

    legend = _("Booking update")
    fields = Fields(IBookingInfo).select('send_update', 'update_subject', 'update_message')

    checker_fieldname = 'send_update'

    weight = 30

    def update_widgets(self, prefix=None, use_form_mode=True):
        super().update_widgets(prefix, use_form_mode)
        send_update = self.widgets.get('send_update')
        if send_update is not None:
            send_update.value = ()
        theater = get_parent(self.context, IMovieTheater)
        templates = IMailTemplates(theater)
        values = get_booking_message_values(self.context, self.request, self)
        # generate formatted messages
        subject = self.widgets.get('update_subject')
        if (subject is not None) and not subject.value:
            template = getattr(templates, 'update_subject')
            if template:
                subject.value = template.format(**values)
        message = self.widgets.get('update_message')
        if (message is not None) and not message.value:
            template = getattr(templates, 'update_template')
            if template:
                message.value = template.format(**values)


@viewlet_config(name='update-group.help',
                context=IBookingInfo, layer=IAdminLayer, view=BookingEditFormUpdateGroup,
                manager=IHelpViewletManager, weight=10)
class BookingEditFormUpdateGroupHelp(AlertMessage):
    """Booking edit form update group help"""

    status = 'info'
    _message = _("Update message can be sent to booking recipient when a booking "
                 "is modified after being validated.")


@adapter_config(name='reminder.group',
                required=(IBookingInfo, IAdminLayer, BookingPropertiesEditForm),
                provides=IGroup)
class BookingEditFormReminderGroup(FormGroupChecker):
    """Booking edit form reminder group"""

    def __new__(cls, context, request, form):
        if context.status != BOOKING_STATUS.ACCEPTED.value:
            return None
        theater = IMovieTheater(context)
        settings = IMovieTheaterSettings(theater)
        if not settings.reminder_delay:
            return None
        reminder_date = (context.session.start_date -
                         datetime.timedelta(days=settings.reminder_delay))
        if reminder_date < tztime(datetime.datetime.now(timezone.utc)):
            return None
        return FormGroupChecker.__new__(cls)

    legend = _("Booking reminder")
    fields = Fields(IBookingInfo).select('send_reminder', 'reminder_subject',
                                         'reminder_message')
    checker_fieldname = 'send_reminder'

    weight = 40

    def update_widgets(self, prefix=None, use_form_mode=True):
        super().update_widgets(prefix, use_form_mode)
        values = get_booking_message_values(self.context, self.request, self)
        theater = get_parent(self.context, IMovieTheater)
        templates = IMailTemplates(theater)
        # generate formatted messages
        subject = self.widgets.get('reminder_subject')
        if (subject is not None) and not subject.value:
            template = getattr(templates, 'reminder_subject')
            if template:
                subject.value = template.format(**values)
        message = self.widgets.get('reminder_message')
        if (message is not None) and not message.value:
            template = getattr(templates, 'reminder_template')
            if template:
                message.value = template.format(**values)


@subscriber(IDataExtractedEvent, form_selector=BookingEditFormReminderGroup)
def extract_booking_reminder_data(event):
    """Booking reminder data extract event"""
    data = event.data
    if data.get('send_reminder') and not data.get('reminder_message'):
        event.form.widgets.errors += (Invalid(_("You must set message content to be able to "
                                                "send a reminder message!")),)


@viewlet_config(name='reminder-group.help',
                context=IBookingInfo, layer=IAdminLayer, view=BookingEditFormReminderGroup,
                manager=IHelpViewletManager, weight=10)
class BookingEditFormReminderGroupHelp(AlertMessage):
    """Booking edit form reminder group help"""

    status = 'info'

    @property
    def message(self):
        translate = self.request.localizer.translate
        theater = IMovieTheater(self.context)
        settings = IMovieTheaterSettings(theater)
        return translate(_("Reminder message will be sent to booking recipient {} hours before "
                           "session begin date...")).format(settings.reminder_delay)


@view_config(name='reset-quotation.json',
             context=IBookingInfo, request_type=IPyAMSLayer, request_method='POST',
             renderer='json', xhr=True,
             permission=MANAGE_BOOKING_PERMISSION)
def reset_quotation(request):
    """Reset booking quotation"""
    booking = IBookingInfo(request.context)
    booking.get_quotation(force_refresh=True)
    translate = request.localizer.translate
    return {
        'status': "success",
        'message': translate(_("Quotation has been reset!"))
    }


@view_config(name='reset-quotation.pdf',
             context=IBookingInfo, request_type=IPyAMSLayer,
             permission=MANAGE_BOOKING_PERMISSION)
def reset_quotation_pdf(request):
    """Get and store new PDF from booking quotation"""
    booking = IBookingInfo(request.context)
    quotation = booking.get_quotation(force_refresh=True)
    if isinstance(quotation, tuple):
        filename, quotation = quotation
    else:
        filename = 'quotation.pdf'
    return Response(content_type='application/pdf',
                    content_disposition=f'inline; filename={filename}',
                    body=quotation)


@view_config(name='preview-quotation.pdf',
             context=IBookingInfo, request_type=IPyAMSLayer,
             permission=MANAGE_BOOKING_PERMISSION)
def preview_quotation_pdf(request):
    """Preview new PDF from booking quotation"""
    booking = IBookingInfo(request.context)
    params = request.params
    quotation = booking.get_quotation(force_refresh=True,
                                      store=False,
                                      nb_participants=params.get('nb_participants'),
                                      nb_accompanists=params.get('nb_accompanists'),
                                      nb_free_accompanists=params.get('nb_free_accompanists'),
                                      price=params.get('price'),
                                      ratio=params.get('ratio'))
    if isinstance(quotation, tuple):
        filename, quotation = quotation
    else:
        filename = 'quotation.pdf'
    return Response(content_type='application/pdf',
                    content_disposition=f'inline; filename={filename}',
                    body=quotation)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface

from pyams_app_msc.shared.catalog.portlet.skin.interfaces import ICatalogViewItemsPortletCalendarRendererSettings, \
    ICatalogViewItemsPortletPanelsRendererSettings
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces.form import IFormFields, IGroup
from pyams_portal.zmi.interfaces import IPortletRendererSettingsEditForm
from pyams_utils.adapter import adapter_config
from pyams_zmi.form import FormGroupChecker, FormGroupSwitcher
from pyams_zmi.interfaces import IAdminLayer

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


#
# Catalog view items portlet calendar renderer settings edit form
#

@adapter_config(required=(ICatalogViewItemsPortletCalendarRendererSettings, IAdminLayer,
                          IPortletRendererSettingsEditForm),
                provides=IFormFields)
def catalog_view_items_calendar_renderer_settings_fields(context, request, form):
    return Fields(Interface)


@adapter_config(name='css-classes',
                required=(ICatalogViewItemsPortletCalendarRendererSettings, IAdminLayer,
                          IPortletRendererSettingsEditForm),
                provides=IGroup)
class CatalogViewItemsPortletCalendarRendererClassesSettingsGroup(FormGroupSwitcher):
    """Catalog view items portlet calendar renderer CSS classes settings group"""

    legend = _("CSS classes")

    fields = Fields(ICatalogViewItemsPortletCalendarRendererSettings).select('filters_css_class',
                                                                             'calendar_css_class')
    weight = 5


@adapter_config(name='base',
                required=(ICatalogViewItemsPortletCalendarRendererSettings, IAdminLayer,
                          IPortletRendererSettingsEditForm),
                provides=IGroup)
class CatalogViewItemsPortletCalendarRendererBaseSettingsGroup(Group):
    """Catalog view items portlet calendar renderer settings group"""

    legend = _("Sessions display")

    fields = Fields(ICatalogViewItemsPortletCalendarRendererSettings).select('sessions_weeks')
    weight = 10


#
# Catalog view items portlet panels renderer settings edit form
#

@adapter_config(name='css-classes',
                required=(ICatalogViewItemsPortletPanelsRendererSettings, IAdminLayer,
                          IPortletRendererSettingsEditForm),
                provides=IGroup)
class CatalogViewItemsPortletPanelsRendererClassesSettingsGroup(FormGroupSwitcher):
    """Catalog view items portlet panels renderer CSS classes settings group"""

    legend = _("CSS classes")

    fields = Fields(ICatalogViewItemsPortletPanelsRendererSettings).select('filters_css_class',
                                                                           'results_css_class',
                                                                           'first_panel_css_class',
                                                                           'panels_css_class')
    weight = 5


@adapter_config(name='properties',
                required=(ICatalogViewItemsPortletPanelsRendererSettings, IAdminLayer,
                          IPortletRendererSettingsEditForm),
                provides=IGroup)
class CatalogViewItemsPortletPanelsRendererSettingsPropertiesGroup(FormGroupChecker):
    """Catalog view items portlet panels renderer settings group"""

    fields = Fields(ICatalogViewItemsPortletPanelsRendererSettings).select('display_sessions', 'sessions_weeks',
                                                                           'display_free_seats')
    weight = 25



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from datetime import datetime, time, timedelta, timezone

from pyramid.events import subscriber
from pyramid.httpexceptions import HTTPBadRequest, HTTPNotFound
from pyramid.view import view_config
from zope.copy import copy
from zope.interface import Invalid
from zope.lifecycleevent import ObjectModifiedEvent

from pyams_app_msc.feature.booking import IBookingContainer
from pyams_app_msc.feature.planning.interfaces import IPlanning, IPlanningTarget, ISession, IWfPlanningTarget
from pyams_app_msc.interfaces import MANAGE_PLANNING_PERMISSION, VIEW_BOOKING_PERMISSION, VIEW_PLANNING_PERMISSION
from pyams_app_msc.shared.catalog.interfaces import ICatalogEntryInfo, IWfCatalogEntry
from pyams_app_msc.shared.theater.interfaces import IMovieTheater, IMovieTheaterSettings
from pyams_app_msc.shared.theater.interfaces.room import ICinemaRoomContainer
from pyams_content_api.feature.json.interfaces import IJSONExporter
from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_skin.interfaces.view import IModalEditForm
from pyams_skin.schema.button import ActionButton, SubmitButton
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_workflow.interfaces import IWorkflowVersions
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle, IModalEditFormButtons
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@ajax_form_config(name='add-session.html',
                  context=IWfPlanningTarget, layer=IPyAMSLayer,
                  permission=MANAGE_PLANNING_PERMISSION)
class SessionAddForm(AdminModalAddForm):
    """Session add form"""

    subtitle = _("New session")
    legend = _("New session properties")

    fields = Fields(ISession).omit('duration', 'archived')
    content_factory = ISession

    _edit_permission = MANAGE_PLANNING_PERMISSION

    def update_widgets(self, prefix=None):
        entry_info = IWfCatalogEntry(self.context, None)
        if entry_info is not None:
            audiences = self.request.params.get(f'{self.prefix}widgets.audiences')
            if audiences is None:
                for audience in (entry_info.audiences or ()):
                    self.request.GET.add(f'{self.prefix}widgets.audiences', audience)
        super().update_widgets(prefix)
        widgets = self.widgets
        params = self.request.params
        theater = get_parent(self.context, IMovieTheater)
        room = None
        if theater is not None:
            room = ICinemaRoomContainer(theater).get(params.get('room'))
        start_date = widgets.get('start_date')
        if start_date is not None:
            if 'start' in params:
                value = datetime.fromisoformat(params.get('start'))
            elif room is not None:
                start_time = room.start_time
                value = datetime.now(timezone.utc).replace(hour=start_time.hour, minute=start_time.minute)
            else:
                value = datetime.now(timezone.utc)
            if (value.time() == time(0, 0, 0)) and (room is not None):  # month mode
                start_time = room.start_time
                value = value.replace(hour=start_time.hour, minute=start_time.minute)
            start_date.value = tztime(value).isoformat()
        end_date = widgets.get('end_date')
        if (end_date is not None) and not end_date.value:
            entry = IWfCatalogEntry(self.context, None)
            if start_date is not None:
                settings = IMovieTheaterSettings(theater, None)
                if entry is not None:
                    end = datetime.fromisoformat(start_date.value)
                    entry_info = ICatalogEntryInfo(entry)
                    if entry_info.duration:
                        end += timedelta(minutes=entry_info.duration)
                    elif settings is not None:
                        end += timedelta(minutes=settings.default_session_duration)
                    if settings is not None:
                        end += timedelta(minutes=settings.session_duration_delta)
                    end_date.value = end.isoformat()
                elif settings is not None:
                    end = datetime.fromisoformat(start_date.value) + \
                        timedelta(minutes=settings.default_session_duration)
                    end_date.value = end.isoformat()
            if room is not None:
                room_widget = widgets.get('room')
                if room_widget is not None:
                    room_widget.value = room.__name__
                capacity = widgets.get('capacity')
                if capacity is not None:
                    capacity.value = room.capacity

    def add(self, obj):
        IPlanning(self.context).add_session(obj)


@adapter_config(required=(IPlanningTarget, IAdminLayer, SessionAddForm),
                provides=IFormTitle)
def planning_add_form_title(context, request, form):
    theater = get_parent(context, IMovieTheater)
    if theater is context:
        return get_object_label(theater, request, form)
    return TITLE_SPAN_BREAK.format(
        get_object_label(theater, request, form),
        get_object_label(context, request, form))


@adapter_config(required=(IPlanningTarget, IAdminLayer, SessionAddForm),
                provides=IAJAXFormRenderer)
class SessionAddFormAJAXRenderer(ContextRequestViewAdapter):
    """Session add form AJAX renderer"""

    def render(self, changes):
        if changes is None:
            return None
        exporter = self.request.registry.getMultiAdapter((changes, self.request), IJSONExporter)
        return {
            'status': 'success',
            'callbacks': [{
                'callback': 'MyAMS.msc.calendar.addEventCallback',
                'options': {
                    'event': exporter.to_json(with_edit_info=True,
                                              edit_context=IPlanning(self.context))
                }
            }]
        }


def can_view_bookings(form):
    """Check if session bookings can be viewed"""
    return form.request.has_permission(VIEW_BOOKING_PERMISSION, context=form.context)

    
def can_delete_session(form):
    """Check if session can be deleted or modified"""
    return form.request.has_permission(MANAGE_PLANNING_PERMISSION, context=form.context)


class ISessionPropertiesEditFormButtons(IModalEditFormButtons):
    """Session properties edit form buttons"""

    bookings = ActionButton(name='bookings',
                            title=_("Bookings"),
                            condition=can_view_bookings)

    delete = SubmitButton(name='delete',
                          title=_("Delete"),
                          condition=can_delete_session)


@ajax_form_config(name='properties.html',
                  context=ISession, layer=IPyAMSLayer,
                  permission=VIEW_PLANNING_PERMISSION)
class SessionPropertiesEditForm(AdminModalEditForm):
    """Session properties edit form"""

    subtitle = _("Session planning")
    legend = _("Session properties")

    fields = Fields(ISession).omit('duration')
    buttons = Buttons(ISessionPropertiesEditFormButtons).select('bookings', 'apply',
                                                                'delete', 'close')

    def update_actions(self):
        super().update_actions()
        bookings = self.actions.get('bookings')
        if bookings is not None:
            if self.context.bookable:
                bookings.add_class('btn-info mr-auto')
                bookings.href = absolute_url(self.context, self.request, 'bookings.html')
                bookings.modal_target = True
            else:
                bookings.add_class('hidden')
        delete = self.actions.get('delete')
        if delete is not None:
            delete.add_class('btn-danger')
            bookings = IBookingContainer(self.context)
            if len(bookings.values()) > 0:
                delete.href = 'MyAMS.msc.session.confirmDelete'

    @handler(buttons['delete'])
    def handle_delete(self, action):
        event_id = self.context.__name__
        planning = IPlanning(self.context)
        del planning[event_id]
        self.finished_state.update({
            'action': action,
            'changes': self.context,
            'event_id': event_id,
            'status': 'deleted'
        })

    @handler(buttons['apply'])
    def handle_apply(self, action):
        super().handle_apply(self, action)


@adapter_config(required=(ISession, IAdminLayer, IModalEditForm),
                provides=IFormTitle)
def session_edit_form_title(context, request, form):
    """Session edit form title"""
    theater = get_parent(context, IMovieTheater)
    target = get_parent(context, IPlanningTarget)
    if target is theater:
        translate = request.localizer.translate
        return TITLE_SPAN_BREAK.format(
            get_object_label(theater, request, form),
            translate(_("Out of catalog activity")))
    versions = IWorkflowVersions(target, None)
    if versions is not None:
        target = versions.get_version(-1)
    return TITLE_SPAN_BREAK.format(
        get_object_label(theater, request, form),
        get_object_label(target, request, form))


@subscriber(IDataExtractedEvent, form_selector=SessionAddForm)
@subscriber(IDataExtractedEvent, form_selector=SessionPropertiesEditForm)
def handle_properties_session_data(event):
    """Handle properties session data"""
    data = event.data
    form = event.form
    context = form.context
    theater = get_parent(context, IMovieTheater)
    # check room capacity
    capacity = data.get('capacity')
    if capacity:
        room = ICinemaRoomContainer(theater).get(data.get('room'))
        if (room is not None) and (capacity > room.capacity):
            form.widgets.errors += (Invalid(_("You can't define a session gauge higher than "
                                              "the room's capacity!")),)
        # check session validated seats
        booking = IBookingContainer(context, None)
        if (booking is not None) and (capacity < booking.get_confirmed_seats()):
            form.widgets.errors += (Invalid(_("The number of validated seats is already higher "
                                              "than this gauge!")),)


@adapter_config(required=(ISession, IAdminLayer, SessionPropertiesEditForm),
                provides=IAJAXFormRenderer)
class SessionPropertiesEditFormAJAXRenderer(ContextRequestViewAdapter):
    """Session properties edit form AJAX renderer"""

    def render(self, changes):
        if not changes:
            return
        exporter = self.request.registry.getMultiAdapter((self.context, self.request), IJSONExporter)
        result = {
            'status': self.request.localizer.translate(self.view.success_message),
            'callbacks': [{
                'callback': 'MyAMS.msc.calendar.editEventCallback',
                'options': {
                    'event': exporter.to_json(with_edit_info=True)
                }
            }]
        }
        return result


@view_config(name='clone-event.json',
             context=ISession, request_type=IPyAMSLayer,
             permission=MANAGE_PLANNING_PERMISSION,
             renderer='json')
def clone_event(request):
    """Clone event"""
    old_session = request.context
    new_session = copy(old_session)
    planning = get_parent(old_session, IPlanning)
    planning.add_session(new_session)
    exporter = request.registry.getMultiAdapter((new_session, request), IJSONExporter)
    return {
        'status': 'success',
        'event': exporter.to_json(with_edit_info=True)
    }


@adapter_config(name='delete',
                required=(ISession, IAdminLayer, SessionPropertiesEditForm),
                provides=IAJAXFormRenderer)
class SessionPropertiesEditFormDeleteActionRenderer(ContextRequestViewAdapter):
    """Session properties edit form delete action renderer"""

    def render(self, changes):
        event_id = self.view.finished_state.get('event_id')
        if not event_id:
            return None
        return {
            'status': 'success',
            'message': self.request.localizer.translate(self.view.success_message),
            'handle_json': True,
            'callbacks': [{
                'callback': 'MyAMS.msc.calendar.deleteEventCallback',
                'options': {
                    'event_id': event_id,
                    'room': self.context.room
                }
            }]
        }


@view_config(name='update-event.json',
             context=ISession, request_type=IPyAMSLayer,
             permission=MANAGE_PLANNING_PERMISSION,
             renderer='json')
def update_event(request):
    """Update event duration after calendar resize"""
    params = request.params
    room = params.get('room')
    start = params.get('start')
    end = params.get('end')
    if not (room and start):
        raise HTTPBadRequest()
    session = request.context
    # check gauge and room seats
    translate = request.localizer.translate
    theater = IMovieTheater(session)
    target_room = ICinemaRoomContainer(theater, {}).get(room)
    if target_room is None:
        request.response.status_code = HTTPNotFound.code
        return {
            'status': 'error',
            'message': translate(_("Unknown room!"))
        }
    container = IBookingContainer(session)
    if target_room.capacity < container.get_confirmed_seats():
        request.response.status_code = HTTPBadRequest.code
        return {
            'status': 'error',
            'messagebox': translate(_("Target room doesn't have enough capacity "
                                      "for confirmed seats of this session!"))
        }
    try:
        # check start date
        start_date = datetime.fromisoformat(start)
        if start_date.time() == time(0, 0):
            # drop in month view -> just set time
            start_date = start_date.replace(hour=session.start_date.hour,
                                            minute=session.start_date.minute)
        # check end date
        if end:
            end_date = datetime.fromisoformat(end)
        else:
            end_date = start_date + session.duration
        # set session attributes
        reduced_capacity = False
        increased_capacity = False
        session.start_date = tztime(start_date)
        session.end_date = tztime(end_date)
        if room != session.room:
            session.room = room
            if target_room.capacity < session.capacity:
                session.capacity = target_room.capacity
                reduced_capacity = True
            elif target_room.capacity > session.capacity:
                session.capacity = target_room.capacity
                increased_capacity = True
        request.registry.notify(ObjectModifiedEvent(session))
    except ValueError:
        request.response.status_code = HTTPBadRequest.code
        return {
            'status': 'error',
            'message': translate(_("Bad parameters"))
        }
    else:
        exporter = request.registry.getMultiAdapter((session, request), IJSONExporter)
        result = {
            'status': 'success',
            'event': exporter.to_json(with_edit_info=True)
        }
        if reduced_capacity:
            result['messagebox'] = {
                'status': 'warning',
                'title': translate(_("Session updated")),
                'message': translate(_("Session capacity has been reduced due to "
                                       "lower room capacity!"))
            }
        elif increased_capacity:
            result['messagebox'] = {
                'status': 'warning',
                'title': translate(_("Session updated")),
                'message': translate(_("Session capacity has been increased to match new room "
                                       "capacity; please check session properties if capacity "
                                       "has to be reduced!"))
            }
        return result



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyams_app_msc.shared.theater import IMovieTheater
from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_security_views.zmi.interfaces import IObjectSecurityMenu
from pyams_utils.adapter import NullAdapter
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer


@viewlet_config(name='managers-restrictions.menu',
                context=IMovieTheater, layer=IAdminLayer,
                manager=IObjectSecurityMenu, weight=200,
                permission=MANAGE_TOOL_PERMISSION)
class MovieTheaterManagersRestrictionsMenu(NullAdapter):
    """Disabled movie theater manager restrictions menu"""


@viewlet_config(name='contributors-restrictions.menu',
                context=IMovieTheater, layer=IAdminLayer,
                manager=IObjectSecurityMenu, weight=210,
                permission=MANAGE_TOOL_PERMISSION)
class MovieTheaterContributorsRestrictionsMenu(NullAdapter):
    """Disabled movie theater contributor restrictions menu"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from collections import OrderedDict
from enum import Enum

from zope.annotation.interfaces import IAttributeAnnotatable
from zope.container.constraints import contains
from zope.container.interfaces import IContainer
from zope.interface import Attribute, Invalid, invariant
from zope.schema import Bool, Choice, Datetime, Int, Text, TextLine
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_app_msc.shared.theater.interfaces.price import PRICES_VOCABULARY
from pyams_file.schema import FileField
from pyams_scheduler.interfaces import ITask
from pyams_security.schema import PrincipalField
from pyams_utils.schema import HTMLField

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


class BOOKING_STATUS(Enum):
    """Booking status"""
    WAITING = 'waiting'
    CANCELLED = 'cancelled'
    REFUSED = 'refused'
    OPTION = 'option'
    ACCEPTED = 'accepted'


BOOKING_CANCELLABLE_STATUS = {
    BOOKING_STATUS.WAITING.value,
    BOOKING_STATUS.REFUSED.value,
    BOOKING_STATUS.OPTION.value,
    BOOKING_STATUS.ACCEPTED.value
}

BOOKING_OPTIONABLE_STATUS = {
    BOOKING_STATUS.WAITING.value,
    BOOKING_STATUS.CANCELLED.value,
    BOOKING_STATUS.REFUSED.value,
    BOOKING_STATUS.ACCEPTED.value
}

BOOKING_REFUSABLE_STATUS = {
    BOOKING_STATUS.WAITING.value,
    BOOKING_STATUS.OPTION.value
}

BOOKING_ACCEPTABLE_STATUS = {
    BOOKING_STATUS.WAITING.value,
    BOOKING_STATUS.CANCELLED.value,
    BOOKING_STATUS.REFUSED.value,
    BOOKING_STATUS.OPTION.value
}


BOOKING_STATUS_LABEL = OrderedDict((
    (BOOKING_STATUS.WAITING, _("Waiting")),
    (BOOKING_STATUS.CANCELLED, _("Cancelled")),
    (BOOKING_STATUS.REFUSED, _("Refused")),
    (BOOKING_STATUS.OPTION, _("Optional")),
    (BOOKING_STATUS.ACCEPTED, _("Accepted"))
))


BOOKING_STATUS_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v.value, title=t)
    for v, t in BOOKING_STATUS_LABEL.items()
])


REQUESTED_BOOKING_STATUS = {
    BOOKING_STATUS.WAITING.value,
    BOOKING_STATUS.OPTION.value,
    BOOKING_STATUS.ACCEPTED.value
}

OCCUPIED_BOOKING_STATUS = {
    BOOKING_STATUS.ACCEPTED.value
}


BOOKING_SESSIONS_VOCABULARY = 'msc.booking_sessions'


class IBookingInfo(IAttributeAnnotatable):
    """Booking information interface

    This interface defines properties of a single booking.
    """

    creator = PrincipalField(title=_("Creator"),
                             description=_("Name of the principal who created the booking"),
                             required=True)

    recipient = PrincipalField(title=_("Recipient"),
                               description=_("Name of the principal for which the booking is done"),
                               required=True)

    def get_recipient(self, with_info=True):
        """Recipient principal getter"""

    status = Choice(title=_("Status"),
                    vocabulary=BOOKING_STATUS_VOCABULARY,
                    required=False,
                    default=BOOKING_STATUS.WAITING.value)

    nb_participants = Int(title=_("Participants"),
                          description=_("Number of participants seats reserved for this session"),
                          required=True,
                          min=0)

    nb_accompanists = Int(title=_("Accompanists"),
                          description=_("Total number of accompanists seats reserved for this session"),
                          required=True,
                          min=0)

    nb_seats = Int(title=_("Total participants"),
                   description=_("Total number of participants reserved for this session"),
                   readonly=True)
    
    nb_free_accompanists = Int(title=_("Free accompanists"),
                               description=_("Number of free accompanists seats reserved for this session"),
                               required=True,
                               min=0,
                               default=0)
    
    @invariant
    def check_nb_accompanists(self):
        if self.nb_free_accompanists > self.nb_accompanists:
            raise Invalid(_("Number of free accompanists can't be higher than total number of accompanists!"))

    accompanying_ratio = Int(title=_("Accompanying ratio"),
                             description=_("You can set an integer number which will define the count of "
                                           "participants for which one accompanying person will have free access"),
                             default=0,
                             min=0,
                             required=False)

    nb_groups = Int(title=_("Groups count"),
                    description=_("Number of groups or classrooms attending this session"),
                    required=True,
                    min=1,
                    default=1)

    price = Choice(title=_("Price"),
                   description=_("Price applied to this booking"),
                   vocabulary=PRICES_VOCABULARY,
                   required=False)

    def get_price(self):
        """Price instance getter"""

    cultural_pass = Bool(title=_("Cultural pass"),
                         description=_("Check this option if payment is done using cultural pass"),
                         required=True,
                         default=False)

    comments = Text(title=_("Comments"),
                    description=_("These comments where added by booking recipient"),
                    required=False)

    notepad = Text(title=_("Notepad"),
                   description=_("These comments are for internal use only"),
                   required=False)

    archived = Bool(title=_("Archived"),
                    description=_("A booking is archived automatically after session end and "
                                  "can't be modified anymore"),
                    required=True,
                    default=False)

    session = Attribute("Booking session")

    session_index = Attribute("Booking session index value")

    def set_session(self, new_session):
        """Update booking session"""

    def get_quotation(self, force_refresh=False, store=True, **kwargs):
        """Create quotation document from booking information"""

    quotation_number = TextLine(title=_("Quotation number"),
                                description=_("This is the reference number of the quotation"),
                                required=False)

    quotation_message = Text(title=_("Quotation message"),
                             description=_("This message is added to quotation as an information message"),
                             required=False)

    quotation = FileField(title=_("Quotation"),
                          description=_("This quotation was attached to booking confirmation"),
                          required=False)

    send_update = Bool(title=_("Send update message?"),
                       description=_("If 'yes', a message will be sent to the recipient "),
                       required=True,
                       default=False)

    update_subject = TextLine(title=_("Update subject"),
                              description=_("Subject of update message sent to the recipient"),
                              required=False)

    update_message = HTMLField(title=_("Update message"),
                               description=_("Update message will be sent to the recipient if booking "
                                             "properties are changed..."),
                               required=False)

    send_reminder = Bool(title=_("Send reminder?"),
                         description=_("If 'yes', a reminder message will be sent to the recipient a few "
                                       "days before the session take place; delay before session is defined "
                                       "into theater settings"),
                         required=False,
                         default=True)

    reminder_subject = TextLine(title=_("Reminder subject"),
                                description=_("Subject of reminder message sent to the recipient"),
                                required=False)

    reminder_message = HTMLField(title=_("Reminder message"),
                                 description=_("Reminder message will be sent to the recipient a few days before "
                                               "the session take place; delay before session is defined into "
                                               "theater settings"),
                                 required=False)

    reminder_date = Datetime(title=_("Reminder date"),
                             description=_("Date and time at which reminder message was sent"),
                             required=False)

    def send_reminder_message(self):
        """Send reminder message"""


class IBookingAcceptInfo(IBookingInfo):
    """Booking accept info interface"""

    accepted = Bool(title=_("Accept booking"),
                    required=True,
                    default=False)


class IBaseBookingWorkflowInfo(IBookingInfo):
    """Base booking workflow info interface"""

    created = Datetime(title=_("Creation date"),
                       description=_("Date at which the booking was created"),
                       required=False)

    notify_recipient = Bool(title=_("Notify recipient?"),
                            description=_("If 'yes', a notification message will be sent to the recipient"),
                            required=False,
                            default=True)

    notify_subject = TextLine(title=_("Notification subject"),
                              description=_("Subject of notification message sent to the recipient"),
                              required=False)

    notify_message = HTMLField(title=_("Notification message"),
                               description=_("Notification message sent to the recipient"),
                               required=False)


class IAcceptedBookingWorkflowInfo(IBaseBookingWorkflowInfo):
    """Accepted booking workflow interface"""

    include_quotation = Bool(title=_("Include quotation"),
                             description=_("Include quotation document into notification message"),
                             required=False,
                             default=True)

    quotation_message = Text(title=_("Quotation message"),
                             description=_("This message will be added to quotation"),
                             required=False)


#
# Booking container
#

BOOKING_CONTAINER_KEY = 'msc.booking'


class IBookingContainer(IContainer):
    """Booking manager interface

    This interface is an :py:class:`IBookingInfo` container.
    """

    contains(IBookingInfo)

    session = Attribute("Booking session")

    def append(self, booking):
        """Add booking to container"""

    def get_requested_seats(self):
        """Get number of requested seats"""

    def get_waiting_seats(self):
        """Get number of waiting seats"""

    def get_confirmed_seats(self):
        """Get number of confirmed reserved seats"""

    free_seats = Int(title=_("Free seats"),
                     description=_("Number of free seats for this session"),
                     readonly=True)

    def get_seats(self, display_mode):
        """Get total number of seats according to given display mode"""


class IBookingTarget(IAttributeAnnotatable):
    """Booking target marker interface"""


class IBookingReminderTask(ITask):
    """Booking reminder task interface"""


class IBookingArchiverTask(ITask):
    """Booking archiver task interface"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_app_msc.shared.catalog.interfaces module

"""

from collections import OrderedDict
from datetime import datetime
from enum import Enum

from zope.annotation.interfaces import IAttributeAnnotatable
from zope.container.constraints import contains
from zope.interface import Attribute, Interface
from zope.schema import Choice, Int, List, Text, TextLine
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_app_msc.component.duration.schema import DurationField
from pyams_app_msc.shared.theater.interfaces.audience import AUDIENCES_VOCABULARY
from pyams_content.feature.filter import IFilter
from pyams_content.feature.filter.interfaces import FILTER_SORTING, MANUAL_FILTER_SORTING_VOCABULARY
from pyams_content.shared.common.interfaces import IBaseSharedTool, ISharedContent
from pyams_content.shared.common.interfaces.types import IWfTypedSharedContentPortalContext, \
    VISIBLE_DATA_TYPES_VOCABULARY
from pyams_i18n.interfaces import BASE_LANGUAGES_VOCABULARY_NAME
from pyams_portal.interfaces import IPortalContext
from pyams_utils.schema import HTMLField

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


CATALOG_ENTRY_INFORMATION_KEY = 'msc.entry_info'


class ICatalogEntryInfo(Interface):
    """Catalog entry additional information interface"""

    description = HTMLField(title=_("Description"),
                            required=False)

    release_year = Choice(title=_("Release year"),
                          values=range(datetime.today().year, 1900, -1),
                          required=False)

    original_country = TextLine(title=_("Original country"),
                                required=False)

    original_title = TextLine(title=_("Original title"),
                              required=False)

    original_language = Choice(title=_("Original language"),
                               vocabulary=BASE_LANGUAGES_VOCABULARY_NAME,
                               required=False)

    producer = TextLine(title=_("Producers"),
                        required=False)

    writer = TextLine(title=_("Writers"),
                      required=False)

    director = TextLine(title=_("Director"),
                        required=False)

    composer = TextLine(title=_("Composer"),
                        required=False)

    actors = TextLine(title=_("Main actors"),
                      required=False)

    awards = Text(title=_("Awards"),
                  required=False)

    duration = DurationField(title=_("Duration"),
                             description=_("Duration, in minutes"),
                             required=False)

    synopsis = Text(title=_("Synopsis"),
                    required=False)


CATALOG_ENTRY_CONTENT_TYPE = 'catalog'
CATALOG_ENTRY_CONTENT_NAME = _("Catalog entry")


class IWfCatalogEntryAddInfo(Interface):
    """Catalog entry adding info interface"""

    title = TextLine(title=_("Title"),
                     description=_("Movie or activity title; you can select an entry from TMDB database, "
                                   "or enter your own title"),
                     required=True)


class CATALOG_ENTRY_SESSION_REQUEST_MODE(Enum):
    """Catalog entry session request mode"""
    INHERIT = 'inherit'
    ENABLED = 'enabled'
    DISABLED = 'disabled'


CATALOG_ENTRY_SESSION_REQUEST_MODE_LABELS = OrderedDict((
    (CATALOG_ENTRY_SESSION_REQUEST_MODE.INHERIT.value, _("(inherit from theater settings)")),
    (CATALOG_ENTRY_SESSION_REQUEST_MODE.ENABLED.value, _("Enable new sessions requests")),
    (CATALOG_ENTRY_SESSION_REQUEST_MODE.DISABLED.value, _("Disable new sessions requests")),
))


CATALOG_ENTRY_SESSION_REQUEST_MODE_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v, title=t)
    for v, t in CATALOG_ENTRY_SESSION_REQUEST_MODE_LABELS.items()
])


class IWfCatalogEntry(IWfTypedSharedContentPortalContext):
    """Catalog entry interface"""

    data_type = Choice(title=_("Activity type"),
                       required=True,
                       vocabulary=VISIBLE_DATA_TYPES_VOCABULARY)

    tmdb_movie_id = Int(title=_("TMDB movie ID"),
                        description=_("Movie ID in TMDB database"),
                        required=False)

    audiences = List(title=_("Selected audiences"),
                     description=_("List of audiences selected for this activity"),
                     value_type=Choice(vocabulary=AUDIENCES_VOCABULARY),
                     required=False)

    allow_session_request = Choice(title=_("New sessions requests"),
                                   description=_('When enabled, external partners will be allowed to '
                                                 'send requests for new sessions of this activity'),
                                   vocabulary=CATALOG_ENTRY_SESSION_REQUEST_MODE_VOCABULARY,
                                   required=True,
                                   default=CATALOG_ENTRY_SESSION_REQUEST_MODE.INHERIT.value)

    def can_request_session(self):
        """Check if a new session request can be created for this activity"""


class ICatalogEntry(ISharedContent):
    """Workflow managed catalog entry interface"""


CATALOG_MANAGER_KEY = 'msc.catalog'


class ICatalogManager(IBaseSharedTool, IPortalContext):
    """Catalog manager interface"""

    contains('.ICatalogEntry')

    shared_content_type = Attribute("Shared data content type name")


class ICatalogManagerTarget(IAttributeAnnotatable):
    """Catalog manager target marker interface"""


class IAudienceFilter(IFilter):
    """Audience filter interface"""


class IDurationFilter(IFilter):
    """Activity duration filter interface"""

    sorting_mode = Choice(title=_("Sorting mode"),
                          description=_("Filter entries sorting mode"),
                          vocabulary=MANUAL_FILTER_SORTING_VOCABULARY,
                          default=FILTER_SORTING.MANUAL.value,
                          required=True)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_content.component.paragraph import IParagraphContainerTarget
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_utils.adapter import NullAdapter
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu

__docformat__ = 'restructuredtext'


@viewlet_config(name='paragraphs-associations.menu',
                context=IParagraphContainerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=110,
                permission=VIEW_SYSTEM_PERMISSION)
class ParagraphsAssociationsMenu(NullAdapter):
    """Paragraphs associations menu"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.schema.interfaces import IInt

__docformat__ = 'restructuredtext'


class IDurationField(IInt):
    """Duration field marker interface"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.feature.tmdb.interfaces import ITMDBServiceClientTarget
from pyams_app_msc.shared.catalog.interfaces import CATALOG_ENTRY_CONTENT_NAME, CATALOG_ENTRY_CONTENT_TYPE, \
    CATALOG_ENTRY_INFORMATION_KEY, CATALOG_ENTRY_SESSION_REQUEST_MODE, ICatalogEntry, ICatalogEntryInfo, IWfCatalogEntry
from pyams_app_msc.shared.theater.interfaces import IMovieTheater, IMovieTheaterSettings
from pyams_content.component.illustration.interfaces import IIllustrationTarget, ILinkIllustrationTarget
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget
from pyams_content.component.thesaurus.interfaces import ITagsTarget, IThemesTarget
from pyams_content.feature.preview.interfaces import IPreviewTarget
from pyams_content.feature.review.interfaces import IReviewTarget
from pyams_content.shared.common import SharedContent, WfSharedContent
from pyams_content.shared.common.interfaces import ISharedContent, IWfSharedContent
from pyams_content.shared.common.types import WfTypedSharedContentMixin
from pyams_portal.interfaces import IPortalContext
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.traversing import get_parent


@factory_config(IWfCatalogEntry)
@factory_config(IWfSharedContent, name=CATALOG_ENTRY_CONTENT_TYPE)
@implementer(ITMDBServiceClientTarget,
             IIllustrationTarget, ILinkIllustrationTarget,
             IParagraphContainerTarget, ITagsTarget, IThemesTarget,
             IPortalContext, IReviewTarget, IPreviewTarget)
class WfCatalogEntry(WfSharedContent, WfTypedSharedContentMixin):
    """Catalog entry"""

    content_type = CATALOG_ENTRY_CONTENT_TYPE
    content_name = CATALOG_ENTRY_CONTENT_NAME
    content_intf = IWfCatalogEntry

    tmdb_movie_id = FieldProperty(IWfCatalogEntry['tmdb_movie_id'])
    audiences = FieldProperty(IWfCatalogEntry['audiences'])
    allow_session_request = FieldProperty(IWfCatalogEntry['allow_session_request'])

    def can_request_session(self):
        """Check if a new session request can be made for this activity"""
        if self.allow_session_request == CATALOG_ENTRY_SESSION_REQUEST_MODE.INHERIT.value:
            theater = get_parent(self, IMovieTheater)
            settings = IMovieTheaterSettings(theater, None)
            return settings.allow_session_request if settings is not None else False
        return self.allow_session_request == CATALOG_ENTRY_SESSION_REQUEST_MODE.ENABLED.value


@factory_config(ICatalogEntryInfo)
class CatalogEntryInfo(Persistent, Contained):
    """Catalog entry persistent information"""

    description = FieldProperty(ICatalogEntryInfo['description'])
    release_year = FieldProperty(ICatalogEntryInfo['release_year'])
    original_country = FieldProperty(ICatalogEntryInfo['original_country'])
    original_title = FieldProperty(ICatalogEntryInfo['original_title'])
    original_language = FieldProperty(ICatalogEntryInfo['original_language'])
    producer = FieldProperty(ICatalogEntryInfo['producer'])
    writer = FieldProperty(ICatalogEntryInfo['writer'])
    director = FieldProperty(ICatalogEntryInfo['director'])
    composer = FieldProperty(ICatalogEntryInfo['composer'])
    actors = FieldProperty(ICatalogEntryInfo['actors'])
    awards = FieldProperty(ICatalogEntryInfo['awards'])
    duration = FieldProperty(ICatalogEntryInfo['duration'])
    synopsis = FieldProperty(ICatalogEntryInfo['synopsis'])


@adapter_config(required=ICatalogEntryInfo,
                provides=IViewContextPermissionChecker)
def catalog_entry_info_permission_checker(context):
    """Catalog entry info permission checker"""
    entry = get_parent(context, IWfCatalogEntry)
    return IViewContextPermissionChecker(entry)


@adapter_config(required=IWfCatalogEntry,
                provides=ICatalogEntryInfo)
def catalog_entry_info(context):
    """Catalog entry information adapter"""
    return get_annotation_adapter(context, CATALOG_ENTRY_INFORMATION_KEY, ICatalogEntryInfo)


@factory_config(ICatalogEntry)
@factory_config(ISharedContent, name=CATALOG_ENTRY_CONTENT_TYPE)
class CatalogEntry(SharedContent):
    """Workflow managed catalog entry"""

    content_type = CATALOG_ENTRY_CONTENT_TYPE
    content_name = CATALOG_ENTRY_CONTENT_NAME



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from hypatia.interfaces import ICatalog
from persistent import Persistent
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectRemovedEvent
from zope.location.interfaces import ISublocations
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
from zope.traversing.interfaces import ITraversable

from pyams_app_msc.interfaces import MANAGE_THEATER_PERMISSION
from pyams_app_msc.shared.theater import ICinemaAudienceContainerTarget, IMovieTheater
from pyams_app_msc.shared.theater.interfaces.audience import AUDIENCES_VOCABULARY, CINEMA_AUDIENCE_CONTAINER_KEY, \
    ICinemaAudience, ICinemaAudienceContainer
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.container import BTreeOrderedContainer
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config


@factory_config(ICinemaAudience)
class CinemaAudience(Persistent, Contained):
    """Cinema audience persistent class"""

    active = FieldProperty(ICinemaAudience['active'])
    name = FieldProperty(ICinemaAudience['name'])
    age_min = FieldProperty(ICinemaAudience['age_min'])
    age_max = FieldProperty(ICinemaAudience['age_max'])
    comment = FieldProperty(ICinemaAudience['comment'])
    contact = FieldProperty(ICinemaAudience['contact'])
    notepad = FieldProperty(ICinemaAudience['notepad'])


@subscriber(IObjectAddedEvent, context_selector=ICinemaAudience)
def handle_new_audience(event):
    """Handle new audience"""
    catalog = get_utility(ICatalog)
    index = catalog.get('facets')
    if index is not None:
        audience_name = f'audience:{event.object.__name__}'
        facets = index.facets
        if audience_name not in facets:
            facets.add(audience_name)
            index.facets = facets


@subscriber(IObjectRemovedEvent, context_selector=ICinemaAudience)
def handle_removed_audience(event):
    """Handle removed audience"""
    catalog = get_utility(ICatalog)
    index = catalog.get('facets')
    if index is not None:
        audience_name = f'audience:{event.object.__name__}'
        facets = index.facets
        if audience_name in facets:
            facets.remove(audience_name)
            index.facets = facets


@factory_config(ICinemaAudienceContainer)
class CinemaAudienceContainer(BTreeOrderedContainer):
    """Cinema audience container persistent class"""

    def get_active_items(self):
        """Active items iterator"""
        yield from filter(lambda x: x.active, self.values())


@adapter_config(required=ICinemaAudienceContainerTarget,
                provides=ICinemaAudienceContainer)
def cinema_audience_container(context):
    """Cinema audience container adapter"""
    return get_annotation_adapter(context, CINEMA_AUDIENCE_CONTAINER_KEY,
                                  ICinemaAudienceContainer,
                                  name='++audience++')


@adapter_config(name='audience',
                required=ICinemaAudienceContainerTarget,
                provides=ITraversable)
class CinemaAudienceContainerTraverser(ContextAdapter):
    """Cinema audience container traverser"""

    def traverse(self, name, furtherPath):
        """Traverse target to audiences container"""
        return ICinemaAudienceContainer(self.context)


@adapter_config(name='audiences',
                required=ICinemaAudienceContainerTarget,
                provides=ISublocations)
class CinemaAudienceSublocations(ContextAdapter):
    """Cinema audiences sub-locations"""

    def sublocations(self):
        """Sub-locations getter"""
        yield from ICinemaAudienceContainer(self.context).values()


@adapter_config(required=ICinemaAudience,
                provides=IViewContextPermissionChecker)
class CinemaAudiencePermissionChecker(ContextAdapter):
    """Cinema audience permission checker"""

    edit_permission = MANAGE_THEATER_PERMISSION


@vocabulary_config(name=AUDIENCES_VOCABULARY)
class CinemaAudiencesVocabulary(SimpleVocabulary):
    """Cinema audiences vocabulary"""

    def __init__(self, context):
        terms = []
        theater = get_parent(context, IMovieTheater)
        if theater is not None:
            terms = [
                SimpleTerm(item.__name__, title=item.name)
                for item in ICinemaAudienceContainer(theater).get_active_items()
            ]
        super().__init__(terms)


def get_audience_contact(theater, audience_id):
    """Get contact email for given audience ID"""
    contact_email = None
    if audience_id:
        audience = ICinemaAudienceContainer(theater).get(audience_id)
        if audience is not None:
            contact = audience.contact
            if contact is not None:
                contact_email = f'{contact.name} <{contact.email_address}>' if contact.email_address else None
    if not contact_email:
        contact_email = theater.contact_email
    return contact_email



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from ZODB.interfaces import IConnection
from persistent import Persistent
from zope.container.contained import Contained
from zope.location.interfaces import ISublocations
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
from zope.traversing.interfaces import ITraversable

from pyams_app_msc.interfaces import MANAGE_THEATER_PERMISSION
from pyams_app_msc.shared.theater import ICinemaPriceContainerTarget, IMovieTheater
from pyams_app_msc.shared.theater.interfaces.price import CINEMA_PRICE_CONTAINER_KEY, ICinemaPrice, \
    ICinemaPriceContainer, PRICES_VOCABULARY
from pyams_catalog.utils import index_object
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.container import BTreeOrderedContainer
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config


@factory_config(ICinemaPrice)
class CinemaPrice(Persistent, Contained):
    """Cinema price persistent class"""

    active = FieldProperty(ICinemaPrice['active'])
    name = FieldProperty(ICinemaPrice['name'])
    participant_price = FieldProperty(ICinemaPrice['participant_price'])
    accompanying_ratio = FieldProperty(ICinemaPrice['accompanying_ratio'])
    accompanying_price = FieldProperty(ICinemaPrice['accompanying_price'])
    comment = FieldProperty(ICinemaPrice['comment'])
    notepad = FieldProperty(ICinemaPrice['notepad'])


@factory_config(ICinemaPriceContainer)
class CinemaPriceContainer(BTreeOrderedContainer):
    """Cinema price container persistent class"""

    def get_active_items(self):
        """Active items iterator"""
        yield from filter(lambda x: x.active, self.values())


@adapter_config(required=ICinemaPriceContainerTarget,
                provides=ICinemaPriceContainer)
def cinema_price_container(context):
    """Cinema price container adapter"""
    return get_annotation_adapter(context, CINEMA_PRICE_CONTAINER_KEY,
                                  ICinemaPriceContainer,
                                  name='++price++')


@adapter_config(name='price',
                required=ICinemaPriceContainerTarget,
                provides=ITraversable)
class CinemaPriceContainerTraverser(ContextAdapter):
    """Cinema price container traverser"""

    def traverse(self, name, furtherPath):
        """Traverse target to prices container"""
        return ICinemaPriceContainer(self.context)


@adapter_config(name='prices',
                required=ICinemaPriceContainerTarget,
                provides=ISublocations)
class CinemaPriceSublocations(ContextAdapter):
    """Cinema prices sub-locations"""

    def sublocations(self):
        """Sub-locations getter"""
        yield from ICinemaPriceContainer(self.context).values()


@adapter_config(required=ICinemaPrice,
                provides=IViewContextPermissionChecker)
class CinemaPricePermissionChecker(ContextAdapter):
    """Cinema price permission checker"""

    edit_permission = MANAGE_THEATER_PERMISSION


@vocabulary_config(name=PRICES_VOCABULARY)
class CinemaPricesVocabulary(SimpleVocabulary):
    """Cinema prices vocabulary"""

    def __init__(self, context):
        terms = []
        theater = get_parent(context, IMovieTheater)
        if theater is not None:
            terms = sorted([
                SimpleTerm(item.__name__, title=item.name)
                for item in ICinemaPriceContainer(theater).get_active_items()
            ], key=lambda x: x.title)
        super().__init__(terms)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyams_app_msc.interfaces import VIEW_THEATER_PERMISSION
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_content.interfaces import MANAGE_REFERENCE_TABLE_PERMISSION, MANAGE_SITE_TREE_PERMISSION
from pyams_content.reference import IReferenceTable
from pyams_viewlet.manager import viewletmanager_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import INavigationViewletManager, ISiteManagementMenu
from pyams_zmi.zmi.viewlet.menu import SiteManagementMenu


@viewletmanager_config(name='site-manager.menu',
                       layer=IAdminLayer,
                       manager=INavigationViewletManager, weight=200,
                       provides=ISiteManagementMenu,
                       permission=MANAGE_SITE_TREE_PERMISSION)
class MSCSiteManagementMenu(SiteManagementMenu):
    """MSC site management menu"""


@viewletmanager_config(name='site-manager.menu',
                       context=IReferenceTable, layer=IAdminLayer,
                       manager=INavigationViewletManager, weight=200,
                       provides=ISiteManagementMenu,
                       permission=MANAGE_REFERENCE_TABLE_PERMISSION)
class MSCReferenceTableManagementMenu(SiteManagementMenu):
    """MSC reference table site management menu"""


@viewletmanager_config(name='site-manager.menu',
                       context=IMovieTheater, layer=IAdminLayer,
                       manager=INavigationViewletManager, weight=200,
                       provides=ISiteManagementMenu,
                       permission=VIEW_THEATER_PERMISSION)
class MSCMovieTheaterSiteManagementMenu(SiteManagementMenu):
    """MSC movie theater site management menu"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.shared.theater import IMailTemplatesTarget
from pyams_app_msc.shared.theater.interfaces.mail import IMailTemplates, MAIL_TEMPLATES_KEY
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config

__docformat__ = 'restructuredtext'


@factory_config(IMailTemplates)
class MailTemplates(Persistent, Contained):
    """Mail templates"""

    send_copy_to_sender = FieldProperty(IMailTemplates['send_copy_to_sender'])
    cancel_subject = FieldProperty(IMailTemplates['cancel_subject'])
    cancel_template = FieldProperty(IMailTemplates['cancel_template'])
    refuse_subject = FieldProperty(IMailTemplates['refuse_subject'])
    refuse_template = FieldProperty(IMailTemplates['refuse_template'])
    option_subject = FieldProperty(IMailTemplates['option_subject'])
    option_template = FieldProperty(IMailTemplates['option_template'])
    accept_subject = FieldProperty(IMailTemplates['accept_subject'])
    accept_template = FieldProperty(IMailTemplates['accept_template'])
    update_subject = FieldProperty(IMailTemplates['update_subject'])
    update_template = FieldProperty(IMailTemplates['update_template'])
    reminder_subject = FieldProperty(IMailTemplates['reminder_subject'])
    reminder_template = FieldProperty(IMailTemplates['reminder_template'])


@adapter_config(required=IMailTemplatesTarget,
                provides=IMailTemplates)
def mail_templates(context):
    """Mail templates adapter"""
    return get_annotation_adapter(context, MAIL_TEMPLATES_KEY, IMailTemplates)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyramid.httpexceptions import HTTPNotFound
from ZODB.interfaces import IConnection
from zope.container.btree import BTreeContainer
from zope.interface import classImplements
from zope.location.interfaces import ISublocations
from zope.traversing.interfaces import ITraversable

from pyams_app_msc.feature.planning.interfaces import IPlanning, IPlanningTarget, IWfPlanningTarget, \
    PLANNING_ANNOTATION_KEY
from pyams_app_msc.shared.catalog import CatalogEntry, WfCatalogEntry
from pyams_app_msc.shared.theater import Theater
from pyams_catalog.utils import index_object
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.traversing import get_parent


classImplements(Theater, IPlanningTarget)

classImplements(CatalogEntry, IPlanningTarget)
classImplements(WfCatalogEntry, IWfPlanningTarget)


@factory_config(IPlanning)
class Planning(BTreeContainer):
    """Planning persistent class"""

    def add_session(self, session):
        """Add session to planning"""
        IConnection(self).add(session)
        key = ICacheKeyValue(session)
        self[key] = session
        index_object(session)


@adapter_config(required=IPlanningTarget,
                provides=IPlanning)
def planning_factory(context):
    """Planning factory adapter"""
    return get_annotation_adapter(context, PLANNING_ANNOTATION_KEY, IPlanning,
                                  name='++planning++')


@adapter_config(name='planning',
                required=IPlanningTarget,
                provides=ITraversable)
class PlanningTraverser(ContextAdapter):
    """Planning traverser"""

    def traverse(self, name, furtherPath=None):
        """Traverse to inner planning"""
        planning = IPlanning(self.context, None)
        if planning is not None:
            return planning
        raise HTTPNotFound()


@adapter_config(required=IWfPlanningTarget,
                provides=IPlanning)
def workflow_planning_factory(context):
    """Workflow managed planning factory adapter"""
    parent = get_parent(context, IPlanningTarget, allow_context=False)
    return IPlanning(parent, None)


@adapter_config(name='planning',
                required=IWfPlanningTarget,
                provides=ITraversable)
def workflow_planning_traverser(context):
    """Workflow managed content planning traverser"""
    parent = get_parent(context, IPlanningTarget, allow_context=False)
    return ITraversable(parent, None)


@adapter_config(name='planning',
                required=IPlanningTarget,
                provides=ISublocations)
class PlanningSublocations(ContextAdapter):
    """Planning sublocations adapter"""

    def sublocations(self):
        """Planning target sub-locations getter"""
        planning = IPlanning(self.context, None)
        if planning is not None:
            yield from planning.values()



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_app_msc.shared.catalog import ICatalogEntryInfo, IWfCatalogEntry
from pyams_content_es.interfaces import IDocumentIndexInfo
from pyams_utils.adapter import adapter_config

__docformat__ = 'restructuredtext'


@adapter_config(name='catalog_info',
                required=IWfCatalogEntry,
                provides=IDocumentIndexInfo)
def catalog_entry_index_info(context):
    """Catalog entry index info"""
    result = {}
    audiences = context.audiences
    if audiences:
        result['audiences'] = context.audiences
    entry_info = ICatalogEntryInfo(context, None)
    if entry_info is None:
        return result
    for field_name in (context.field_names or ()):
        value = getattr(entry_info, field_name)
        if not value:
            continue
        result[field_name] = value
    return {
        'catalog_info': result
    }



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface
from zope.schema import TextLine, Choice, Bool
from pyams_file.interfaces.thumbnail import THUMBNAILERS_VOCABULARY_NAME

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


class IMovieTheaterCarouselPortletRendererSettings(Interface):
    """Movie theater carousel portlet renderer settings interface"""

    css_class = TextLine(title=_("CSS class"),
                         description=_("Carousel container CSS class"),
                         required=False,
                         default='carousel')

    cards_css_class = TextLine(title=_("Cards CSS class"),
                               description=_("Inner cards CSS class"),
                               required=False,
                               default='col-md-4 mb-3')

    thumb_selection = Choice(title=_("Images selection"),
                             description=_("Carousel can use responsive selections, but you can "
                                           "also force selection of another specific selection"),
                             vocabulary=THUMBNAILERS_VOCABULARY_NAME,
                             required=False)

    automatic_slide = Bool(title=_("Automatic sliding"),
                           description=_("If 'no', sliding will only be activated manually"),
                           required=True,
                           default=True)

    fade_effect = Bool(title=_("Fade effect"),
                       description=_("If 'yes', slide to slide animation will use a fade effect "
                                     "instead of lateral sliding"),
                       required=True,
                       default=False)

    display_controls = Bool(title=_("Display controls"),
                            description=_("If 'yes', display arrows to navigate between slides"),
                            required=True,
                            default=False)

    enable_touch = Bool(title=_("Enable swiping"),
                        description=_("If 'no', touch events will be disabled on touchscreens"),
                        required=True,
                        default=True)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_app_msc.feature.tmdb interfaces module

"""

from zope.interface import Interface, Invalid, invariant
from zope.schema import Bool, Choice, Int, Object, TextLine, URI

from pyams_i18n.interfaces import ISO_LANGUAGES_VOCABULARY_NAME
from pyams_utils.interfaces.proxy import IProxyInfo

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


TMDB_SEARCH_ROUTE = 'pyams_msc.tmdb.search'
'''TMDB search API route'''

TMDB_SEARCH_PATH = '/api/msc/tmdb'
'''TMDB search API default path'''

TMDB_CONFIGURATION_KEY = 'msc.tmdb.configuration'
'''TMDB configuration annotations key'''


class ITMDBConfiguration(Interface):
    """TMDB configuration interface"""

    enabled = Bool(title=_("Enable TMDB configuration"),
                   required=True,
                   default=False)

    service_url = URI(title=_("Service URL"),
                      description=_("Main TMDB services base URL"),
                      required=False,
                      default="https://api.themoviedb.org")

    proxy_info = Object(title=_("Proxy information"),
                        description=_("Enter information if a proxy is required"),
                        required=False,
                        schema=IProxyInfo)

    def get_proxy_info(self, request=None, protocol=None):
        """Proxy information getter"""

    api_key = TextLine(title=_("API key"),
                       description=_("API authentication key (v3)"),
                       required=False)

    bearer_token = TextLine(title=_("Bearer token"),
                            description=_("Bearer authentication token (v4)"),
                            required=False)

    @invariant
    def check_api_key(self):
        """API key checker"""
        if self.enabled and not (self.api_key or self.bearer_token):
            raise Invalid(_("You must define an API key or a Bearer token to enable TMDB configuration!"))

    language = Choice(title=_("Language"),
                      description=_("Primary language used for TMDB API"),
                      vocabulary=ISO_LANGUAGES_VOCABULARY_NAME,
                      required=False,
                      default='en-US')

    movie_search_path = TextLine(title=_("Movie search path"),
                                 description=_("Path to TMDB movie search API"),
                                 required=False,
                                 default='/3/search/movie')

    movie_details_path = TextLine(title=_("Movie details path"),
                                  description=_("Path to TMDB movie details API, including {movie_id} argument"),
                                  required=False,
                                  default='/3/movie/{movie_id}')

    movie_credits_path = TextLine(title=_("Movie credits path"),
                                  description=_("Path to TMDB movie credits API, including {movie_id} argument"),
                                  required=False,
                                  default='/3/movie/{movie_id}/credits')

    actors_count = Int(title=_("Actors count"),
                       description=_("Number of actors "),
                       required=False,
                       default=4)

    movie_images_path = TextLine(title=_("Movie images path"),
                                 description=_("Path to TMDB movie images API, including {movie_id} argument"),
                                 required=False,
                                 default='/3/movie/{movie_id}/images')

    movie_videos_path = TextLine(title=_("Movie videos path"),
                                 description=_("Path to TMDB movie videos API, including {movie_id} argument"),
                                 required=False,
                                 default='/3/movie/{movie_id}/videos')

    images_url = URI(title=_("Images URL"),
                     description=_("TMDB images base URL, including {size} and {image_id} arguments"),
                     required=False,
                     default="https://image.tmdb.org/t/p/{size}{image_id}")

    download_main_poster = Bool(title=_("Download main poster?"),
                                description=_("If 'no', movie poster will not be downloaded on activity creation"),
                                required=True,
                                default=True)

    main_poster_size = Choice(title=_("Main poster size"),
                              description=_("This is the size of poster image downloaded from TMDB"),
                              values=('w92', 'w154', 'w185', 'w342', 'w500', 'w780', 'original'),
                              default='w780',
                              required=False)

    download_pictures = Bool(title=_("Download pictures?"),
                             description=_("If 'no', movie pictures will not be downloaded on activity creation"),
                             required=True,
                             default=True)

    pictures_size = Choice(title=_("Pictures size"),
                           description=_("This is the size of movies images downloaded from TMDB"),
                           values=('w300', 'w780', 'w1280', 'original'),
                           default='w780',
                           required=False)

    download_posters = Bool(title=_("Download posters?"),
                            description=_("If 'no', movie posters will not be downloaded on activity creation"),
                            required=True,
                            default=True)

    posters_size = Choice(title=_("Posters size"),
                          description=_("This is the size of posters images downloaded from TMDB"),
                          values=('w92', 'w154', 'w185', 'w342', 'w500', 'w780', 'original'),
                          default='w780',
                          required=False)

    download_videos = Bool(title=_("Download videos?"),
                           description=_("If 'no', movie videos will not be downloaded on activity creation"),
                           required=True,
                           default=True)

    restrict_videos_language = Bool(title=_("Get videos for selected language only"),
                                    description=_("If 'yes', only videos tagged for selected language will "
                                                  "be selected; otherwise, other videos tagged for any language "
                                                  "may also be selected"),
                                    required=True,
                                    default=True)


class ITMDBService(Interface):
    """TMDB service interface"""

    def find_movies(self, query: str):
        """Find movies matching provided query"""

    def get_movie_info(self, movie_id: int):
        """Get detailed information about movie identified by its TMDB ID"""

    def get_movie_pictures(self, movie_id: int):
        """Get pictures of movie identified by its TMDB ID"""

    def set_activity_info(self, activity):
        """Set TMDB information for provided activity"""


class ITMDBServiceClientTarget(Interface):
    """TMDB service client target marker interface

    This interface is used to tag components which can use TMDB service
    to get information about a given movie.
    """



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from datetime import datetime

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq, Ge, Le
from pyramid.httpexceptions import HTTPBadRequest, HTTPNotFound
from pyramid.view import view_config

from pyams_app_msc.feature.booking.zmi.interfaces import IBookingManagementMenu
from pyams_app_msc.feature.planning import IPlanning
from pyams_app_msc.feature.planning.interfaces import ISession
from pyams_app_msc.feature.planning.zmi import PlanningMenu, PlanningView
from pyams_app_msc.interfaces import VIEW_CATALOG_PERMISSION
from pyams_app_msc.shared.theater import ICinemaRoomContainer, IMovieTheater
from pyams_catalog.query import CatalogResultSet
from pyams_content_api.feature.json import IJSONExporter
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import USE_INTERNAL_API_PERMISSION
from pyams_utils.factory import get_interface_base_name
from pyams_utils.registry import get_utility
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer


@viewlet_config(name='planning.menu',
                context=IMovieTheater, layer=IAdminLayer,
                manager=IBookingManagementMenu, weight=7,
                permission=VIEW_CATALOG_PERMISSION)
class MovieTheaterPlanningMenu(PlanningMenu):
    """Movie theater planning menu"""


@pagelet_config(name='planning.html',
                context=IMovieTheater, layer=IPyAMSLayer,
                permission=VIEW_CATALOG_PERMISSION)
class MovieTheaterCalendarView(PlanningView):
    """Movie theater planning view"""

    def get_context(self):
        """View context getter"""
        return self.context


@view_config(name='get-planning-events.json',
             context=IMovieTheater, request_type=IPyAMSLayer,
             permission=USE_INTERNAL_API_PERMISSION,
             renderer='json')
def get_planning_events(request):
    """Planning events getter"""
    params = request.params
    room = params.get('room')
    start = params.get('start')
    end = params.get('end')
    if not (room and start and end):
        raise HTTPBadRequest()
    theater = IMovieTheater(request.context, None)
    if theater is None:
        raise HTTPNotFound()
    container = ICinemaRoomContainer(theater, None)
    if (container is None) or (room not in container):
        raise HTTPNotFound()
    catalog = get_utility(ICatalog)
    query = And(Eq(catalog['object_types'], get_interface_base_name(ISession)),
                Eq(catalog['planning_room'], room),
                Le(catalog['planning_start_date'], datetime.fromisoformat(end)),
                Ge(catalog['planning_end_date'], datetime.fromisoformat(start)))
    events = []
    for session in CatalogResultSet(CatalogQuery(catalog).query(query)):
        exporter = request.registry.queryMultiAdapter((session, request), IJSONExporter)
        if exporter is not None:
            events.append(exporter.to_json(with_edit_info=True, edit_context=IPlanning(session)))
    return events
